from django.contrib.auth import get_user_model
from django.urls import reverse
from datetime import timedelta
from django.utils import timezone
from rest_framework.test import APITestCase
from djmoney.money import Money

from order.models.item import OrderItem
from order.models.order import Order
from order.enum.status import OrderStatus, PaymentStatus
from user.factories.account import UserAccountFactory
from product.factories.product import ProductFactory
from product.factories.category import ProductCategoryFactory
from country.factories import CountryFactory
from region.factories import RegionFactory
from pay_way.factories import PayWayFactory

User = get_user_model()


class OrderItemFilterTest(APITestCase):
    def setUp(self):
        OrderItem.objects.all().delete()

        self.admin_user = UserAccountFactory(
            email="admin@example.com",
            first_name="Admin",
            last_name="User",
            is_active=True,
            is_staff=True,
            is_superuser=True,
        )
        self.client.force_authenticate(user=self.admin_user)

        self.user1 = UserAccountFactory(
            email="john.doe@example.com", first_name="John", last_name="Doe"
        )
        self.user2 = UserAccountFactory(
            email="jane.smith@example.com", first_name="Jane", last_name="Smith"
        )

        self.category1 = ProductCategoryFactory()
        self.category2 = ProductCategoryFactory()

        self.product1 = ProductFactory(
            sku="PROD001",
            category=self.category1,
            active=True,
            price=Money(25, "EUR"),
        )
        self.product2 = ProductFactory(
            sku="PROD002",
            category=self.category1,
            active=True,
            price=Money(50, "EUR"),
        )
        self.product3 = ProductFactory(
            sku="PROD003",
            category=self.category2,
            active=False,
            price=Money(75, "EUR"),
        )

        self.now = timezone.now()

        country = CountryFactory()
        region = RegionFactory()
        pay_way = PayWayFactory()

        self.order1 = Order.objects.create(
            user=self.user1,
            status=OrderStatus.PENDING,
            payment_status=PaymentStatus.PENDING,
            first_name="John",
            last_name="Doe",
            email="john.doe@example.com",
            country=country,
            region=region,
            pay_way=pay_way,
            street="123 Main St",
            street_number="123",
            city="Test City",
            zipcode="12345",
            shipping_price=Money(10, "EUR"),
            paid_amount=Money(0, "EUR"),
        )
        Order.objects.filter(id=self.order1.id).update(
            created_at=self.now - timedelta(hours=1)
        )

        self.order2 = Order.objects.create(
            user=self.user1,
            status=OrderStatus.COMPLETED,
            payment_status=PaymentStatus.COMPLETED,
            first_name="John",
            last_name="Doe",
            email="john.doe@example.com",
            country=country,
            region=region,
            pay_way=pay_way,
            street="123 Main St",
            street_number="123",
            city="Test City",
            zipcode="12345",
            shipping_price=Money(10, "EUR"),
            paid_amount=Money(100, "EUR"),
        )
        Order.objects.filter(id=self.order2.id).update(
            created_at=self.now - timedelta(days=2)
        )

        self.order3 = Order.objects.create(
            user=self.user2,
            status=OrderStatus.SHIPPED,
            payment_status=PaymentStatus.COMPLETED,
            first_name="Jane",
            last_name="Smith",
            email="jane.smith@example.com",
            country=country,
            region=region,
            pay_way=pay_way,
            street="456 Oak Ave",
            street_number="456",
            city="Test City",
            zipcode="54321",
            shipping_price=Money(15, "EUR"),
            paid_amount=Money(200, "EUR"),
        )
        Order.objects.filter(id=self.order3.id).update(
            created_at=self.now - timedelta(days=1)
        )

        self.item1 = OrderItem.objects.create(
            order=self.order1,
            product=self.product1,
            price=Money(25, "EUR"),
            quantity=2,
            original_quantity=2,
            is_refunded=False,
            refunded_quantity=0,
            notes="Standard item",
        )
        OrderItem.objects.filter(id=self.item1.id).update(
            created_at=self.now - timedelta(hours=1)
        )

        self.item2 = OrderItem.objects.create(
            order=self.order1,
            product=self.product2,
            price=Money(50, "EUR"),
            quantity=1,
            original_quantity=1,
            is_refunded=False,
            refunded_quantity=0,
            notes="",
        )
        OrderItem.objects.filter(id=self.item2.id).update(
            created_at=self.now - timedelta(hours=1)
        )

        self.item3 = OrderItem.objects.create(
            order=self.order2,
            product=self.product1,
            price=Money(25, "EUR"),
            quantity=10,
            original_quantity=10,
            is_refunded=True,
            refunded_quantity=10,
            notes="Customer requested refund",
        )
        OrderItem.objects.filter(id=self.item3.id).update(
            created_at=self.now - timedelta(days=2)
        )

        self.item4 = OrderItem.objects.create(
            order=self.order2,
            product=self.product2,
            price=Money(50, "EUR"),
            quantity=4,
            original_quantity=4,
            is_refunded=False,
            refunded_quantity=2,
            notes="Partial refund processed",
        )
        OrderItem.objects.filter(id=self.item4.id).update(
            created_at=self.now - timedelta(days=2)
        )

        self.item5 = OrderItem.objects.create(
            order=self.order3,
            product=self.product3,
            price=Money(150, "EUR"),
            quantity=1,
            original_quantity=1,
            is_refunded=False,
            refunded_quantity=0,
            notes="",
        )
        OrderItem.objects.filter(id=self.item5.id).update(
            created_at=self.now - timedelta(days=1)
        )

        self.old_item = OrderItem.objects.create(
            order=self.order3,
            product=self.product1,
            price=Money(25, "EUR"),
            quantity=1,
            original_quantity=1,
            is_refunded=False,
            refunded_quantity=0,
            notes="Old item",
        )
        OrderItem.objects.filter(id=self.old_item.id).update(
            created_at=self.now - timedelta(days=10)
        )

    def test_basic_filters(self):
        url = reverse("order-item-list")

        response = self.client.get(url, {"order": self.order1.id})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item2.id, result_ids)

        response = self.client.get(url, {"product": self.product1.id})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 3)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.old_item.id, result_ids)

        response = self.client.get(url, {"is_refunded": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 1)
        self.assertIn(self.item3.id, result_ids)

    def test_product_relationship_filters(self):
        url = reverse("order-item-list")

        product_name = self.product1.safe_translation_getter(
            "name", any_language=True
        )
        if product_name:
            response = self.client.get(url, {"product__name": product_name[:5]})
            self.assertEqual(response.status_code, 200)
            result_ids = [r["id"] for r in response.data["results"]]
            self.assertGreaterEqual(len(result_ids), 1)

        response = self.client.get(url, {"product__sku": "PROD001"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 3)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.old_item.id, result_ids)

        response = self.client.get(
            url, {"product__category": self.category1.id}
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 5)
        self.assertNotIn(self.item5.id, result_ids)

        response = self.client.get(url, {"product__active": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 1)
        self.assertIn(self.item5.id, result_ids)

    def test_order_relationship_filters(self):
        url = reverse("order-item-list")

        response = self.client.get(
            url, {"order__status": OrderStatus.PENDING.value}
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item2.id, result_ids)

        response = self.client.get(
            url, {"order__payment_status": PaymentStatus.COMPLETED.value}
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 4)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item4.id, result_ids)
        self.assertIn(self.item5.id, result_ids)
        self.assertIn(self.old_item.id, result_ids)

        response = self.client.get(url, {"order__user": self.user1.id})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 4)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item2.id, result_ids)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item4.id, result_ids)

        response = self.client.get(url, {"order__user__email": "jane"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.item5.id, result_ids)
        self.assertIn(self.old_item.id, result_ids)

        response = self.client.get(url, {"order__first_name": "john"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 4)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item2.id, result_ids)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item4.id, result_ids)

    def test_price_filters(self):
        url = reverse("order-item-list")

        response = self.client.get(url, {"price_min": "50"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 3)
        self.assertIn(self.item2.id, result_ids)
        self.assertIn(self.item4.id, result_ids)
        self.assertIn(self.item5.id, result_ids)

        response = self.client.get(url, {"price_max": "50"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 5)
        self.assertNotIn(self.item5.id, result_ids)

        response = self.client.get(url, {"price_exact": "25"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 3)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.old_item.id, result_ids)

    def test_quantity_filters(self):
        url = reverse("order-item-list")

        response = self.client.get(url, {"quantity_min": "4"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item4.id, result_ids)

        response = self.client.get(url, {"quantity_max": "2"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 4)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item2.id, result_ids)
        self.assertIn(self.item5.id, result_ids)
        self.assertIn(self.old_item.id, result_ids)

        response = self.client.get(url, {"quantity_exact": "1"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 3)
        self.assertIn(self.item2.id, result_ids)
        self.assertIn(self.item5.id, result_ids)
        self.assertIn(self.old_item.id, result_ids)

        response = self.client.get(url, {"original_quantity_min": "4"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item4.id, result_ids)

    def test_refund_filters(self):
        url = reverse("order-item-list")

        response = self.client.get(url, {"refunded_quantity_min": "1"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item4.id, result_ids)

        response = self.client.get(url, {"has_refunded_quantity": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item4.id, result_ids)

        response = self.client.get(url, {"has_refunded_quantity": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 4)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item2.id, result_ids)
        self.assertIn(self.item5.id, result_ids)
        self.assertIn(self.old_item.id, result_ids)

        response = self.client.get(url, {"is_partially_refunded": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 1)
        self.assertIn(self.item4.id, result_ids)

        response = self.client.get(url, {"is_fully_refunded": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 1)
        self.assertIn(self.item3.id, result_ids)

    def test_notes_filters(self):
        url = reverse("order-item-list")

        response = self.client.get(url, {"has_notes": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 4)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item4.id, result_ids)
        self.assertIn(self.old_item.id, result_ids)

        response = self.client.get(url, {"has_notes": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.item2.id, result_ids)
        self.assertIn(self.item5.id, result_ids)

        response = self.client.get(url, {"notes": "refund"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item4.id, result_ids)

    def test_special_filters(self):
        url = reverse("order-item-list")

        response = self.client.get(url, {"high_value_items": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 1)
        self.assertIn(self.item5.id, result_ids)

        response = self.client.get(url, {"high_value_items": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 5)
        self.assertNotIn(self.item5.id, result_ids)

        response = self.client.get(url, {"bulk_items": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 1)
        self.assertIn(self.item3.id, result_ids)

        response = self.client.get(url, {"bulk_items": "false"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 5)
        self.assertNotIn(self.item3.id, result_ids)

        response = self.client.get(url, {"recent_items": "true"})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 5)
        self.assertNotIn(self.old_item.id, result_ids)

    def test_bulk_filters(self):
        url = reverse("order-item-list")

        order_ids = f"{self.order1.id},{self.order2.id}"
        response = self.client.get(url, {"order_ids": order_ids})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 4)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item2.id, result_ids)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item4.id, result_ids)

        product_ids = f"{self.product1.id},{self.product3.id}"
        response = self.client.get(url, {"product_ids": product_ids})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 4)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item5.id, result_ids)
        self.assertIn(self.old_item.id, result_ids)

        order_statuses = (
            f"{OrderStatus.PENDING.value},{OrderStatus.COMPLETED.value}"
        )
        response = self.client.get(url, {"order_statuses": order_statuses})
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 4)
        self.assertIn(self.item1.id, result_ids)
        self.assertIn(self.item2.id, result_ids)
        self.assertIn(self.item3.id, result_ids)
        self.assertIn(self.item4.id, result_ids)

        response = self.client.get(url, {"order_ids": "invalid,ids"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 0)

    def test_uuid_filter(self):
        url = reverse("order-item-list")

        response = self.client.get(url, {"uuid": str(self.item1.uuid)})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["id"], self.item1.id)

    def test_timestamp_filters(self):
        url = reverse("order-item-list")

        created_after_date = self.now - timedelta(days=3)
        response = self.client.get(
            url, {"created_after": created_after_date.isoformat()}
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 5)
        self.assertNotIn(self.old_item.id, result_ids)

        created_before_date = self.now - timedelta(days=5)
        response = self.client.get(
            url, {"created_before": created_before_date.isoformat()}
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 1)
        self.assertIn(self.old_item.id, result_ids)

    def test_camel_case_filters(self):
        url = reverse("order-item-list")

        response = self.client.get(
            url,
            {
                "product__sku": "PROD001",
                "order__status": OrderStatus.PENDING.value,
                "has_notes": "true",
            },
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 1)
        self.assertIn(self.item1.id, result_ids)

    def test_complex_filter_combinations(self):
        url = reverse("order-item-list")

        response = self.client.get(
            url,
            {
                "order__status": OrderStatus.COMPLETED.value,
                "price_min": "50",
                "has_refunded_quantity": "false",
                "ordering": "-price",
            },
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 0)

        response = self.client.get(
            url,
            {
                "has_refunded_quantity": "true",
                "has_notes": "true",
                "order__payment_status": PaymentStatus.COMPLETED.value,
                "ordering": "-refunded_quantity",
            },
        )
        self.assertEqual(response.status_code, 200)
        result_ids = [r["id"] for r in response.data["results"]]
        self.assertEqual(len(result_ids), 2)

        results = response.data["results"]

        result_ids = {r["id"] for r in results}
        expected_ids = {self.item3.id, self.item4.id}
        self.assertEqual(result_ids, expected_ids)

    def test_filter_with_ordering(self):
        url = reverse("order-item-list")

        response = self.client.get(
            url, {"order__user": self.user1.id, "ordering": "-price"}
        )
        self.assertEqual(response.status_code, 200)
        results = response.data["results"]
        self.assertEqual(len(results), 4)

        user1_items = [self.item1, self.item2, self.item3, self.item4]
        items_by_price_desc = [(item, item.price) for item in user1_items]
        items_by_price_desc.sort(key=lambda x: x[1].amount, reverse=True)

        high_price_ids = {
            items_by_price_desc[0][0].id,
            items_by_price_desc[1][0].id,
        }
        self.assertIn(results[0]["id"], high_price_ids)
        self.assertIn(results[1]["id"], high_price_ids)
        self.assertNotEqual(results[0]["id"], results[1]["id"])

        response = self.client.get(
            url, {"product": self.product1.id, "ordering": "-quantity"}
        )
        self.assertEqual(response.status_code, 200)
        results = response.data["results"]
        self.assertEqual(len(results), 3)

        items_for_product1 = [
            self.item1,
            self.item3,
            self.old_item,
        ]
        items_by_quantity_desc = [
            (item, item.quantity) for item in items_for_product1
        ]
        items_by_quantity_desc.sort(key=lambda x: x[1], reverse=True)

        self.assertEqual(results[0]["id"], items_by_quantity_desc[0][0].id)

    def tearDown(self):
        OrderItem.objects.all().delete()



from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import extend_schema_field
from parler_rest.serializers import TranslatableModelSerializer
from rest_framework import serializers
from rest_framework.relations import PrimaryKeyRelatedField

from core.api.schema import generate_schema_multi_lang
from core.utils.serializers import TranslatedFieldExtended
from country.models import Country
from region.models import Region


@extend_schema_field(generate_schema_multi_lang(Region))
class TranslatedFieldsFieldExtend(TranslatedFieldExtended):
    pass


class RegionSerializer(
    TranslatableModelSerializer, serializers.ModelSerializer[Region]
):
    translations = TranslatedFieldsFieldExtend(shared_model=Region)
    country = PrimaryKeyRelatedField(queryset=Country.objects.all())

    class Meta:
        model = Region
        fields = (
            "translations",
            "alpha",
            "country",
            "sort_order",
            "created_at",
            "updated_at",
            "uuid",
        )
        read_only_fields = (
            "created_at",
            "updated_at",
            "uuid",
        )


class RegionDetailSerializer(RegionSerializer):
    class Meta(RegionSerializer.Meta):
        fields = (*RegionSerializer.Meta.fields,)


class RegionWriteSerializer(
    TranslatableModelSerializer, serializers.ModelSerializer[Region]
):
    translations = TranslatedFieldsFieldExtend(shared_model=Region)
    country = PrimaryKeyRelatedField(queryset=Country.objects.all())

    def validate_alpha(self, value: str) -> str:
        if value and len(value) > 10:
            raise serializers.ValidationError(
                _("Region alpha code should be 10 characters or less.")
            )
        return value.upper() if value else value

    def validate(self, data):
        country = data.get("country")
        alpha = data.get("alpha")

        if country and alpha:
            existing = Region.objects.filter(country=country, alpha=alpha)
            if self.instance:
                existing = existing.exclude(pk=self.instance.pk)

            if existing.exists():
                raise serializers.ValidationError(
                    _(
                        "Region with alpha '{alpha}' already exists for {country_alpha_2}"
                    ).format(
                        alpha=alpha,
                        country_alpha_2=country.alpha_2,
                    )
                )

        return data

    class Meta:
        model = Region
        fields = (
            "translations",
            "alpha",
            "country",
            "sort_order",
        )



# Generated by Django 5.0.4 on 2024-04-24 12:07
import django.contrib.postgres.search
import tinymce.models
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("product", "0012_alter_product_discount_percent"),
    ]

    operations = [
        migrations.AlterField(
            model_name="producttranslation",
            name="description",
            field=tinymce.models.HTMLField(
                blank=True, null=True, verbose_name="Description"
            ),
        ),
        migrations.AlterField(
            model_name="producttranslation",
            name="name",
            field=models.CharField(
                blank=True, max_length=255, null=True, verbose_name="Name"
            ),
        ),
        migrations.AlterField(
            model_name="producttranslation",
            name="search_document",
            field=models.TextField(
                blank=True, default="", verbose_name="Search Document"
            ),
        ),
        migrations.AlterField(
            model_name="producttranslation",
            name="search_document_dirty",
            field=models.BooleanField(
                default=False, verbose_name="Search Document Dirty"
            ),
        ),
        migrations.AlterField(
            model_name="producttranslation",
            name="search_vector",
            field=django.contrib.postgres.search.SearchVectorField(
                blank=True, null=True, verbose_name="Search Vector"
            ),
        ),
        migrations.AlterField(
            model_name="producttranslation",
            name="search_vector_dirty",
            field=models.BooleanField(
                default=False, verbose_name="Search Vector Dirty"
            ),
        ),
    ]



# Generated by Django 5.2.6 on 2025-09-25 21:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('product', '0025_remove_product_product_product_code_ix_and_more'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='productcategory',
            index=models.Index(fields=['tree_id', 'lft'], name='product_productcategory_tr16fb'),
        ),
    ]



from .account import UserAccountFilter
from .address import UserAddressFilter
from .subscription import SubscriptionTopicFilter, UserSubscriptionFilter

__all__ = [
    "SubscriptionTopicFilter",
    "UserAccountFilter",
    "UserAddressFilter",
    "UserSubscriptionFilter",
]



import factory

from vat.models import Vat


class VatFactory(factory.django.DjangoModelFactory):
    value = factory.Faker("random_int", min=1, max=100)

    class Meta:
        model = Vat
        django_get_or_create = ("value",)



import logging
from abc import ABC, abstractmethod
from dataclasses import dataclass
from datetime import date, timedelta
from decimal import Decimal
from enum import Enum
from typing import Any

from django.conf import settings
from django.utils import timezone
from djmoney.money import Money

from country.models import Country
from region.models import Region

logger = logging.getLogger(__name__)


class ShippingMethodType(Enum):
    STANDARD = "STANDARD"
    EXPRESS = "EXPRESS"
    NEXT_DAY = "NEXT_DAY"
    ECONOMY = "ECONOMY"
    INTERNATIONAL = "INTERNATIONAL"
    PICKUP = "PICKUP"


@dataclass
class ShippingOption:
    id: str
    name: str
    method_type: ShippingMethodType
    price: Money
    estimated_delivery_min: int
    estimated_delivery_max: int
    carrier: str
    carrier_service_code: str
    description: str | None = None

    @property
    def estimated_delivery_date_min(self) -> date:
        return timezone.now().date() + timedelta(
            days=self.estimated_delivery_min
        )

    @property
    def estimated_delivery_date_max(self) -> date:
        return timezone.now().date() + timedelta(
            days=self.estimated_delivery_max
        )


class ShippingCarrier(ABC):
    @abstractmethod
    def get_shipping_options(
        self,
        weight: Decimal,
        dimensions: dict[str, Decimal],
        from_country: Country,
        to_country: Country,
        to_region: Region | None = None,
        to_postal_code: str | None = None,
        **kwargs,
    ) -> list[ShippingOption]:
        pass

    @abstractmethod
    def create_shipment(
        self,
        order_id: str,
        shipping_option_id: str,
        address_data: dict[str, Any],
        **kwargs,
    ) -> tuple[bool, dict[str, Any]]:
        pass

    @abstractmethod
    def get_tracking_info(self, tracking_number: str) -> dict[str, Any]:
        pass


class FedExCarrier(ShippingCarrier):
    def __init__(self):
        self.api_key = settings.FEDEX_API_KEY
        self.account_number = settings.FEDEX_ACCOUNT_NUMBER

        # @TODO - In a real implementation, we would import and use FedEx SDK

    def get_shipping_options(
        self,
        weight: Decimal,
        dimensions: dict[str, Decimal],
        from_country: Country,
        to_country: Country,
        to_region: Region | None = None,
        to_postal_code: str | None = None,
        **kwargs,
    ) -> list[ShippingOption]:
        logger.info(
            "Getting FedEx shipping options",
            extra={
                "weight": str(weight),
                "from_country": from_country.alpha_2,
                "to_country": to_country.alpha_2,
                "to_postal_code": to_postal_code,
            },
        )

        # @TODO - Mock implementation - in real world, we would call FedEx API
        # and parse the response

        options = []

        # For domestic shipments, offer 3 options
        if from_country.alpha_2 == to_country.alpha_2:
            options.append(
                ShippingOption(
                    id="fedex_ground",
                    name="FedEx Ground",
                    method_type=ShippingMethodType.STANDARD,
                    price=Money(
                        amount="12.99", currency=settings.DEFAULT_CURRENCY
                    ),
                    estimated_delivery_min=3,
                    estimated_delivery_max=5,
                    carrier="FedEx",
                    carrier_service_code="GROUND",
                    description="Standard delivery in 3-5 business days",
                )
            )

            options.append(
                ShippingOption(
                    id="fedex_express",
                    name="FedEx Express",
                    method_type=ShippingMethodType.EXPRESS,
                    price=Money(
                        amount="19.99", currency=settings.DEFAULT_CURRENCY
                    ),
                    estimated_delivery_min=2,
                    estimated_delivery_max=3,
                    carrier="FedEx",
                    carrier_service_code="EXPRESS",
                    description="Express delivery in 2-3 business days",
                )
            )

            options.append(
                ShippingOption(
                    id="fedex_overnight",
                    name="FedEx Overnight",
                    method_type=ShippingMethodType.NEXT_DAY,
                    price=Money(
                        amount="29.99", currency=settings.DEFAULT_CURRENCY
                    ),
                    estimated_delivery_min=1,
                    estimated_delivery_max=1,
                    carrier="FedEx",
                    carrier_service_code="OVERNIGHT",
                    description="Next day delivery",
                )
            )
        else:
            # For international shipments, offer international option
            options.append(
                ShippingOption(
                    id="fedex_international",
                    name="FedEx International",
                    method_type=ShippingMethodType.INTERNATIONAL,
                    price=Money(
                        amount="49.99", currency=settings.DEFAULT_CURRENCY
                    ),
                    estimated_delivery_min=5,
                    estimated_delivery_max=7,
                    carrier="FedEx",
                    carrier_service_code="INTERNATIONAL_ECONOMY",
                    description="International delivery in 5-7 business days",
                )
            )

        return options

    def create_shipment(
        self,
        order_id: str,
        shipping_option_id: str,
        address_data: dict[str, Any],
        **kwargs,
    ) -> tuple[bool, dict[str, Any]]:
        try:
            logger.info(
                "Creating FedEx shipment",
                extra={
                    "order_id": order_id,
                    "shipping_option_id": shipping_option_id,
                },
            )

            shipment_data = {
                "tracking_number": f"FX{order_id}12345",
                "label_url": f"https://example.com/labels/fedex_{order_id}.pdf",
                "carrier": "FedEx",
                "service": shipping_option_id,
                "created_at": timezone.now().isoformat(),
                "estimated_delivery": (
                    timezone.now().date() + timedelta(days=3)
                ).isoformat(),
            }

            return True, shipment_data

        except Exception as e:
            logger.error(
                f"FedEx shipment creation failed: {e!s}",
                extra={"order_id": order_id, "error": str(e)},
            )
            return False, {"error": str(e)}

    def get_tracking_info(self, tracking_number: str) -> dict[str, Any]:
        try:
            logger.info(
                "Getting FedEx tracking info",
                extra={"tracking_number": tracking_number},
            )

            # @TODO - Mock response
            current_date = timezone.now().date()

            tracking_data = {
                "tracking_number": tracking_number,
                "status": "IN_TRANSIT",
                "estimated_delivery": (
                    current_date + timedelta(days=2)
                ).isoformat(),
                "carrier": "FedEx",
                "events": [
                    {
                        "timestamp": (
                            current_date - timedelta(days=1)
                        ).isoformat(),
                        "location": "Memphis, TN",
                        "description": "Arrived at FedEx hub",
                        "status": "IN_TRANSIT",
                    },
                    {
                        "timestamp": (
                            current_date - timedelta(days=2)
                        ).isoformat(),
                        "location": "Atlanta, GA",
                        "description": "Shipment picked up",
                        "status": "PICKED_UP",
                    },
                ],
            }

            return tracking_data

        except Exception as e:
            logger.error(
                f"Failed to get FedEx tracking info: {e!s}",
                extra={"tracking_number": tracking_number, "error": str(e)},
            )
            return {"error": str(e)}


class UPSCarrier(ShippingCarrier):
    def __init__(self):
        self.api_key = settings.UPS_API_KEY
        self.account_number = settings.UPS_ACCOUNT_NUMBER

        # @TODO - In a real implementation, we would import and use UPS SDK

    def get_shipping_options(
        self,
        weight: Decimal,
        dimensions: dict[str, Decimal],
        from_country: Country,
        to_country: Country,
        to_region: Region | None = None,
        to_postal_code: str | None = None,
        **kwargs,
    ) -> list[ShippingOption]:
        logger.info(
            "Getting UPS shipping options",
            extra={
                "weight": str(weight),
                "from_country": from_country.alpha_2,
                "to_country": to_country.alpha_2,
                "to_postal_code": to_postal_code,
            },
        )

        options = []

        options.append(
            ShippingOption(
                id="ups_worldwide",
                name="UPS Worldwide Expedited",
                method_type=ShippingMethodType.INTERNATIONAL,
                price=Money(amount="47.99", currency=settings.DEFAULT_CURRENCY),
                estimated_delivery_min=4,
                estimated_delivery_max=6,
                carrier="UPS",
                carrier_service_code="WWEX",
                description="International delivery in 4-6 business days",
            )
        )

        return options

    def create_shipment(
        self,
        order_id: str,
        shipping_option_id: str,
        address_data: dict[str, Any],
        **kwargs,
    ) -> tuple[bool, dict[str, Any]]:
        try:
            logger.info(
                "Creating UPS shipment",
                extra={
                    "order_id": order_id,
                    "shipping_option_id": shipping_option_id,
                },
            )

            # @TODO - Mock response
            shipment_data = {
                "tracking_number": f"1Z{order_id}789",
                "label_url": f"https://example.com/labels/ups_{order_id}.pdf",
                "carrier": "UPS",
                "service": shipping_option_id,
                "created_at": timezone.now().isoformat(),
                "estimated_delivery": (
                    timezone.now().date() + timedelta(days=3)
                ).isoformat(),
            }

            return True, shipment_data

        except Exception as e:
            logger.error(
                f"UPS shipment creation failed: {e!s}",
                extra={"order_id": order_id, "error": str(e)},
            )
            return False, {"error": str(e)}

    def get_tracking_info(self, tracking_number: str) -> dict[str, Any]:
        try:
            logger.info(
                "Getting UPS tracking info",
                extra={"tracking_number": tracking_number},
            )

            # @TODO - Mock response
            current_date = timezone.now().date()

            tracking_data = {
                "tracking_number": tracking_number,
                "status": "IN_TRANSIT",
                "estimated_delivery": (
                    current_date + timedelta(days=1)
                ).isoformat(),
                "carrier": "UPS",
                "events": [
                    {
                        "timestamp": (
                            current_date - timedelta(days=1)
                        ).isoformat(),
                        "location": "Louisville, KY",
                        "description": "Arrived at UPS facility",
                        "status": "IN_TRANSIT",
                    },
                    {
                        "timestamp": (
                            current_date - timedelta(days=2)
                        ).isoformat(),
                        "location": "Nashville, TN",
                        "description": "Shipment picked up",
                        "status": "PICKED_UP",
                    },
                ],
            }

            return tracking_data

        except Exception as e:
            logger.error(
                f"Failed to get UPS tracking info: {e!s}",
                extra={"tracking_number": tracking_number, "error": str(e)},
            )
            return {"error": str(e)}


def get_shipping_carrier(carrier_name: str) -> ShippingCarrier:
    carriers = {
        "fedex": FedExCarrier,
        "ups": UPSCarrier,
    }

    carrier_class = carriers.get(carrier_name.lower())
    if not carrier_class:
        raise ValueError(f"Unsupported shipping carrier: {carrier_name}")

    return carrier_class()


class ShippingService:
    @classmethod
    def get_available_shipping_options(
        cls,
        order_weight: Decimal,
        order_dimensions: dict[str, Decimal],
        from_country: Country,
        to_country: Country,
        to_region: Region | None = None,
        to_postal_code: str | None = None,
        **kwargs,
    ) -> list[ShippingOption]:
        all_options = []

        for carrier_name in ["fedex", "ups"]:
            try:
                carrier = get_shipping_carrier(carrier_name)
                options = carrier.get_shipping_options(
                    weight=order_weight,
                    dimensions=order_dimensions,
                    from_country=from_country,
                    to_country=to_country,
                    to_region=to_region,
                    to_postal_code=to_postal_code,
                    **kwargs,
                )
                all_options.extend(options)
            except Exception as e:
                logger.error(
                    f"Error getting shipping options from {carrier_name}: {e!s}",
                    extra={"error": str(e)},
                )

        return sorted(all_options, key=lambda x: x.price.amount)

    @classmethod
    def create_shipment(
        cls,
        order_id: str,
        carrier: str,
        shipping_option_id: str,
        address_data: dict[str, Any],
        **kwargs,
    ) -> tuple[bool, dict[str, Any]]:
        try:
            carrier_instance = get_shipping_carrier(carrier)
            return carrier_instance.create_shipment(
                order_id=order_id,
                shipping_option_id=shipping_option_id,
                address_data=address_data,
                **kwargs,
            )
        except Exception as e:
            logger.error(
                f"Error creating shipment: {e!s}",
                extra={
                    "order_id": order_id,
                    "carrier": carrier,
                    "error": str(e),
                },
            )
            return False, {"error": str(e)}

    @classmethod
    def get_tracking_info(
        cls, tracking_number: str, carrier: str
    ) -> dict[str, Any]:
        try:
            carrier_instance = get_shipping_carrier(carrier)
            return carrier_instance.get_tracking_info(tracking_number)
        except Exception as e:
            logger.error(
                f"Error getting tracking info: {e!s}",
                extra={
                    "tracking_number": tracking_number,
                    "carrier": carrier,
                    "error": str(e),
                },
            )
            return {"error": str(e)}



from datetime import timedelta

from django.db import models
from django.utils import timezone
from extra_settings.models import Setting


class CartQuerySet(models.QuerySet):
    def active(self):
        abandoned_threshold = Setting.get("CART_ABANDONED_HOURS", default=24)
        cutoff_time = timezone.now() - timedelta(hours=abandoned_threshold)
        return self.filter(last_activity__gte=cutoff_time).exclude(
            items__isnull=True
        )

    def abandoned(self):
        abandoned_threshold = Setting.get("CART_ABANDONED_HOURS", default=24)
        cutoff_time = timezone.now() - timedelta(hours=abandoned_threshold)
        return self.filter(last_activity__lt=cutoff_time)

    def empty(self):
        return self.filter(items__isnull=True)

    def with_items(self):
        return self.filter(items__isnull=False)

    def for_user(self, user):
        if user.is_authenticated:
            return self.filter(user=user)
        return self.none()

    def guest_carts(self):
        return self.filter(user__isnull=True)

    def user_carts(self):
        return self.filter(user__isnull=False)

    def with_totals(self):
        return self.prefetch_related("items__product").annotate(
            total_quantity=models.Sum("items__quantity"),
            unique_items_count=models.Count("items", distinct=True),
        )

    def expired(self, days=30):
        cutoff_date = timezone.now() - timedelta(days=days)
        return self.filter(last_activity__lt=cutoff_date)

    def by_date_range(self, start_date, end_date):
        return self.filter(created_at__date__range=[start_date, end_date])

    def recent(self, days=7):
        cutoff_date = timezone.now() - timedelta(days=days)
        return self.filter(created_at__gte=cutoff_date)

    def by_country(self, country_code):
        return self.filter(user__country__alpha_2=country_code)

    def with_specific_product(self, product):
        return self.filter(items__product=product).distinct()


class CartManager(models.Manager):
    def get_queryset(self) -> CartQuerySet:
        return CartQuerySet(self.model, using=self._db)

    def active(self):
        return self.get_queryset().active()

    def abandoned(self):
        return self.get_queryset().abandoned()

    def empty(self):
        return self.get_queryset().empty()

    def with_items(self):
        return self.get_queryset().with_items()

    def for_user(self, user):
        return self.get_queryset().for_user(user)

    def guest_carts(self):
        return self.get_queryset().guest_carts()

    def user_carts(self):
        return self.get_queryset().user_carts()

    def with_totals(self):
        return self.get_queryset().with_totals()

    def expired(self, days=30):
        return self.get_queryset().expired(days)

    def by_date_range(self, start_date, end_date):
        return self.get_queryset().by_date_range(start_date, end_date)

    def recent(self, days=7):
        return self.get_queryset().recent(days)

    def by_country(self, country_code):
        return self.get_queryset().by_country(country_code)

    def with_specific_product(self, product):
        return self.get_queryset().with_specific_product(product)

    def cleanup_expired(self, days=30):
        expired_carts = self.expired(days)
        count = expired_carts.count()
        expired_carts.delete()
        return count



# Generated by Django 5.0.4 on 2024-04-26 12:52
import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("country", "0004_alter_country_options"),
        ("region", "0004_alter_region_options"),
        ("user", "0007_useraccount_username"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="useraccount",
            name="user_account_trgm_idx",
        ),
        migrations.AddIndex(
            model_name="useraccount",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=[
                    "email",
                    "username",
                    "first_name",
                    "last_name",
                    "phone",
                    "city",
                    "zipcode",
                    "address",
                    "place",
                ],
                name="user_account_trgm_idx",
                opclasses=[
                    "gin_trgm_ops",
                    "gin_trgm_ops",
                    "gin_trgm_ops",
                    "gin_trgm_ops",
                    "gin_trgm_ops",
                    "gin_trgm_ops",
                    "gin_trgm_ops",
                    "gin_trgm_ops",
                    "gin_trgm_ops",
                ],
            ),
        ),
    ]



from decimal import Decimal

from django.contrib.postgres.indexes import BTreeIndex
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_stubs_ext.db.models import TypedModelMeta
from djmoney.models.fields import MoneyField
from djmoney.money import Money

from core.models import SortableModel, TimeStampMixinModel, UUIDModel
from order.managers.item import OrderItemManager


class OrderItem(TimeStampMixinModel, SortableModel, UUIDModel):
    id = models.BigAutoField(primary_key=True)
    order = models.ForeignKey(
        "order.Order",
        related_name="items",
        on_delete=models.CASCADE,
    )
    product = models.ForeignKey(
        "product.Product",
        related_name="order_items",
        on_delete=models.CASCADE,
    )
    price = MoneyField(_("Price"), max_digits=11, decimal_places=2)
    quantity = models.IntegerField(_("Quantity"), default=1)
    original_quantity = models.IntegerField(
        _("Original Quantity"), null=True, blank=True
    )
    is_refunded = models.BooleanField(_("Is Refunded"), default=False)
    refunded_quantity = models.IntegerField(_("Refunded Quantity"), default=0)
    notes = models.TextField(_("Notes"), blank=True, default="")

    objects: OrderItemManager = OrderItemManager()

    class Meta(TypedModelMeta):
        verbose_name = _("Order Item")
        verbose_name_plural = _("Order Items")
        ordering = ["sort_order"]
        indexes = [
            *TimeStampMixinModel.Meta.indexes,
            *SortableModel.Meta.indexes,
            BTreeIndex(fields=["product"], name="order_item_product_ix"),
            BTreeIndex(
                fields=["is_refunded"], name="order_item_is_refunded_ix"
            ),
        ]

    def __str__(self):
        product_name = self.product.safe_translation_getter(
            "name", any_language=True
        )
        return f"Order {self.order.id} - {product_name} x {self.quantity}"

    def clean(self):
        if self.quantity <= 0:
            raise ValidationError(_("Quantity must be greater than 0."))

        if (
            hasattr(self.product, "stock")
            and self.quantity > self.product.stock
            and not self.pk
        ):
            raise ValidationError(
                _("The quantity exceeds the available stock.")
            )

        if self.refunded_quantity > self.quantity:
            raise ValidationError(
                _("Refunded quantity cannot exceed the ordered quantity.")
            )

    @property
    def total_price(self) -> Money:
        return Money(
            amount=self.price.amount * Decimal(self.quantity),
            currency=self.price.currency,
        )

    @property
    def refunded_amount(self) -> Money:
        if self.refunded_quantity == 0:
            return Money(amount=0, currency=self.price.currency)

        return Money(
            amount=self.price.amount * Decimal(self.refunded_quantity),
            currency=self.price.currency,
        )

    @property
    def net_quantity(self) -> int:
        return self.quantity - self.refunded_quantity

    @property
    def net_price(self) -> Money:
        return Money(
            amount=self.price.amount * Decimal(self.net_quantity),
            currency=self.price.currency,
        )

    def save(self, *args, **kwargs):
        if not self.pk and self.original_quantity is None:
            self.original_quantity = self.quantity
        super().save(*args, **kwargs)

    def refund(self, quantity=None):
        refund_qty = quantity if quantity is not None else self.quantity

        if refund_qty <= 0:
            raise ValidationError(
                _(
                    "Invalid refund quantity. Please enter a quantity greater than 0."
                )
            )

        if self.refunded_quantity + refund_qty > self.quantity:
            raise ValidationError(
                _(
                    "Cannot refund more than the ordered quantity. "
                    f"Ordered quantity: {self.quantity}, "
                    f"Refunded quantity: {self.refunded_quantity}, "
                    f"Refund quantity: {refund_qty}"
                )
            )

        if hasattr(self.product, "stock"):
            self.product.stock += refund_qty
            self.product.save(update_fields=["stock"])

        self.refunded_quantity += refund_qty
        if self.refunded_quantity == self.quantity:
            self.is_refunded = True

        self.save(update_fields=["refunded_quantity", "is_refunded"])

        return Money(
            amount=self.price.amount * Decimal(refund_qty),
            currency=self.price.currency,
        )

    def get_ordering_queryset(self):
        return self.order.items.all()



# Generated by Django 5.2.1 on 2025-05-14 20:33

import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('country', '0006_remove_country_country_cou_alpha_2_398383_idx_and_more'),
        ('region', '0005_alter_region_sort_order'),
    ]

    operations = [
        migrations.RenameIndex(
            model_name='region',
            new_name='region_created_at_ix',
            old_name='region_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='region',
            new_name='region_updated_at_ix',
            old_name='region_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='region',
            new_name='region_sort_order_ix',
            old_name='region_sort_order_idx',
        ),
        migrations.AddIndex(
            model_name='region',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['alpha'], name='region_alpha_ix'),
        ),
        migrations.AddIndex(
            model_name='region',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['country'], name='region_country_ix'),
        ),
    ]



# Generated by Django 5.2.1 on 2025-05-16 22:41

import django.contrib.postgres.indexes
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pay_way', '0007_rename_payway_created_at_idx_payway_created_at_ix_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='payway',
            name='configuration',
            field=models.JSONField(blank=True, help_text='Provider-specific configuration (API keys, webhooks, etc.)', null=True, verbose_name='Provider Configuration'),
        ),
        migrations.AddField(
            model_name='payway',
            name='is_online_payment',
            field=models.BooleanField(default=False, help_text='Whether this payment method is processed online', verbose_name='Is Online Payment'),
        ),
        migrations.AddField(
            model_name='payway',
            name='provider_code',
            field=models.CharField(blank=True, default='', help_text="Code used to identify the payment provider in the system (e.g., 'stripe', 'paypal')", max_length=50, verbose_name='Provider Code'),
        ),
        migrations.AddField(
            model_name='payway',
            name='requires_confirmation',
            field=models.BooleanField(default=False, help_text='Whether this payment method requires manual confirmation (e.g., bank transfer)', verbose_name='Requires Confirmation'),
        ),
        migrations.AddField(
            model_name='paywaytranslation',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='Description'),
        ),
        migrations.AddField(
            model_name='paywaytranslation',
            name='instructions',
            field=models.TextField(blank=True, help_text='Instructions for the customer on how to complete payment (e.g., bank transfer details)', null=True, verbose_name='Payment Instructions'),
        ),
        migrations.AlterField(
            model_name='paywaytranslation',
            name='name',
            field=models.CharField(blank=True, choices=[('CREDIT_CARD', 'Credit Card'), ('PAY_ON_DELIVERY', 'Pay On Delivery'), ('PAY_ON_STORE', 'Pay On Store'), ('PAY_PAL', 'PayPal'), ('STRIPE', 'Stripe'), ('BANK_TRANSFER', 'Bank Transfer'), ('APPLE_PAY', 'Apple Pay'), ('GOOGLE_PAY', 'Google Pay')], max_length=50, null=True, verbose_name='Name'),
        ),
        migrations.AddIndex(
            model_name='payway',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['provider_code'], name='pay_way_provider_ix'),
        ),
        migrations.AddIndex(
            model_name='payway',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['is_online_payment'], name='pay_way_online_ix'),
        ),
    ]



from django.conf import settings
from django.test import TestCase

from tag.factories.tag import TagFactory
from tag.factories.tagged_item import TaggedProductFactory
from tag.models.tag import Tag
from product.factories.product import ProductFactory

languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]
default_language = settings.PARLER_DEFAULT_LANGUAGE_CODE


class TagModelTestCase(TestCase):
    def setUp(self):
        self.tag = TagFactory(active=True)
        self.tag.sort_order = 0

    def test_fields(self):
        self.assertTrue(self.tag.active)

    def test_str_representation(self):
        tag_label = (
            self.tag.safe_translation_getter("label", any_language=True)
            or "Unnamed Label"
        )
        self.assertEqual(
            str(self.tag),
            f"{tag_label} ({'Active' if self.tag.active else 'Inactive'})",
        )

    def test_get_ordering_queryset(self):
        Tag.objects.all().delete()

        tags = [
            TagFactory(sort_order=1, active=True),
            TagFactory(sort_order=2, active=True),
            TagFactory(sort_order=3, active=True),
        ]

        for tag in tags:
            tag.save()

        ordered_tags = Tag.objects.all().order_by("sort_order")

        ordered_sort_orders = [tag.sort_order for tag in ordered_tags]

        self.assertEqual(ordered_sort_orders, [1, 2, 3])

    def test_get_usage_count(self):
        self.assertEqual(self.tag.get_usage_count(), 0)

        product1 = ProductFactory()
        product2 = ProductFactory()
        TaggedProductFactory(tag=self.tag, content_object=product1)
        TaggedProductFactory(tag=self.tag, content_object=product2)

        self.assertEqual(self.tag.get_usage_count(), 2)

    def test_get_content_types(self):
        self.assertEqual(self.tag.get_content_types(), "")

        product = ProductFactory()
        TaggedProductFactory(tag=self.tag, content_object=product)

        content_types = self.tag.get_content_types()
        self.assertIn("product.product", content_types)

    def test_active_manager_method(self):
        active_tags = Tag.objects.active()
        self.assertIn(self.tag, active_tags)

        inactive_tag = TagFactory(active=False)
        active_tags = Tag.objects.active()
        self.assertNotIn(inactive_tag, active_tags)

    def test_inactive_manager_method(self):
        inactive_tag = TagFactory(active=False)
        inactive_tags = Tag.objects.inactive()
        self.assertIn(inactive_tag, inactive_tags)

        self.assertNotIn(self.tag, inactive_tags)

    def test_translatable_fields(self):
        for language in languages:
            self.tag.set_current_language(language)
            self.tag.label = f"Test label in {language}"
            self.tag.save()

        for language in languages:
            self.tag.set_current_language(language)
            self.assertIsNotNone(self.tag.label)

    def test_uuid_field(self):
        self.assertIsNotNone(self.tag.uuid)

        other_tag = TagFactory()
        self.assertNotEqual(self.tag.uuid, other_tag.uuid)

    def test_timestamp_fields(self):
        self.assertIsNotNone(self.tag.created_at)
        self.assertIsNotNone(self.tag.updated_at)

        original_updated_at = self.tag.updated_at
        self.tag.active = False
        self.tag.save()

        self.assertNotEqual(original_updated_at, self.tag.updated_at)

    def test_sort_order_default(self):
        new_tag = TagFactory()
        self.assertIsNotNone(new_tag.sort_order)
        self.assertGreaterEqual(new_tag.sort_order, 0)



# Generated by Django 4.2.9 on 2024-02-10 14:39
import uuid

import django.db.models.deletion
import parler.models
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Notification",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "updated_at",
                    models.DateTimeField(auto_now=True, verbose_name="Updated At"),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "link",
                    models.URLField(blank=True, null=True, verbose_name="Link"),
                ),
                (
                    "kind",
                    models.CharField(
                        choices=[
                            ("error", "Error"),
                            ("success", "Success"),
                            ("info", "Info"),
                            ("warning", "Warning"),
                            ("danger", "Danger"),
                        ],
                        default="info",
                        max_length=250,
                        verbose_name="Kind",
                    ),
                ),
            ],
            options={
                "verbose_name": "Notification",
                "verbose_name_plural": "Notifications",
                "ordering": ["-created_at"],
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="NotificationTranslation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                (
                    "title",
                    models.CharField(max_length=250, verbose_name="Title"),
                ),
                ("message", models.TextField(verbose_name="Message")),
            ],
            options={
                "verbose_name": "Notification Translation",
                "db_table": "notification_notification_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="NotificationUser",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "updated_at",
                    models.DateTimeField(auto_now=True, verbose_name="Updated At"),
                ),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "seen",
                    models.BooleanField(default=False, verbose_name="Seen"),
                ),
                (
                    "seen_at",
                    models.DateTimeField(blank=True, null=True, verbose_name="Seen At"),
                ),
                (
                    "notification",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="notification.notification",
                    ),
                ),
            ],
            options={
                "verbose_name": "Notification User",
                "verbose_name_plural": "Notification Users",
                "ordering": ["-notification__created_at"],
            },
        ),
    ]



from django.test import TestCase

from tag.factories.tag import TagFactory
from tag.factories.tagged_item import TaggedProductFactory
from tag.managers import TagManager
from tag.models.tag import Tag
from product.factories.product import ProductFactory


class TestTagManager(TestCase):
    def setUp(self):
        self.active_tag1 = TagFactory(active=True)
        self.active_tag2 = TagFactory(active=True)
        self.inactive_tag = TagFactory(active=False)

    def test_get_queryset_includes_translations(self):
        queryset = Tag.objects.get_queryset()

        self.assertIn("translations", queryset._prefetch_related_lookups)

    def test_active_method(self):
        active_tags = Tag.objects.active()

        self.assertIn(self.active_tag1, active_tags)
        self.assertIn(self.active_tag2, active_tags)

        self.assertNotIn(self.inactive_tag, active_tags)

        self.assertEqual(active_tags.count(), 2)

    def test_inactive_method(self):
        inactive_tags = Tag.objects.inactive()

        self.assertIn(self.inactive_tag, inactive_tags)

        self.assertNotIn(self.active_tag1, inactive_tags)
        self.assertNotIn(self.active_tag2, inactive_tags)

        self.assertEqual(inactive_tags.count(), 1)

    def test_manager_returns_correct_type(self):
        self.assertIsInstance(Tag.objects, TagManager)

        active_queryset = Tag.objects.active()
        self.assertTrue(hasattr(active_queryset, "filter"))
        self.assertTrue(hasattr(active_queryset, "exclude"))

    def test_chaining_methods(self):
        recent_active_tags = (
            Tag.objects.active()
            .filter(created_at__isnull=False)
            .order_by("-created_at")
        )

        self.assertGreaterEqual(recent_active_tags.count(), 0)

        for tag in recent_active_tags:
            self.assertTrue(tag.active)

    def test_select_related_optimization(self):
        queryset = Tag.objects.get_queryset()

        tag = queryset.first()
        if tag:
            self.assertIsNotNone(tag.id)

    def test_prefetch_translations(self):
        TagFactory(active=True)

        queryset = Tag.objects.all()
        retrieved_tag = queryset.first()

        self.assertIsNotNone(retrieved_tag)


class TestTagManagerWithUsage(TestCase):
    def setUp(self):
        self.tag_with_usage = TagFactory(active=True)
        self.tag_without_usage = TagFactory(active=True)

        product = ProductFactory()
        TaggedProductFactory(tag=self.tag_with_usage, content_object=product)

    def test_tags_with_usage_annotation(self):
        tags = Tag.objects.all()

        self.assertEqual(tags.count(), 2)

        for tag in tags:
            usage_count = tag.get_usage_count()
            self.assertIsInstance(usage_count, int)
            self.assertGreaterEqual(usage_count, 0)

    def test_manager_performance_with_large_dataset(self):
        tags = []
        for i in range(10):
            tag = TagFactory(active=(i % 2 == 0))
            tags.append(tag)

        active_tags = Tag.objects.active()
        active_count = active_tags.count()

        self.assertGreaterEqual(active_count, 5)

        inactive_tags = Tag.objects.inactive()
        inactive_count = inactive_tags.count()

        self.assertGreaterEqual(inactive_count, 5)

    def test_empty_queryset_behavior(self):
        Tag.objects.all().delete()

        active_tags = Tag.objects.active()
        self.assertEqual(active_tags.count(), 0)

        inactive_tags = Tag.objects.inactive()
        self.assertEqual(inactive_tags.count(), 0)

    def test_manager_with_custom_filtering(self):
        filtered_active = Tag.objects.active().filter(
            id__in=[self.tag_with_usage.id]
        )

        self.assertEqual(filtered_active.count(), 1)
        self.assertEqual(filtered_active.first(), self.tag_with_usage)

    def test_manager_method_consistency(self):
        all_tags = Tag.objects.all()
        active_tags = Tag.objects.active()
        inactive_tags = Tag.objects.inactive()

        total_expected = active_tags.count() + inactive_tags.count()
        self.assertEqual(all_tags.count(), total_expected)



import logging
from os import listdir, path, remove

from django.conf import settings
from django.core.management.base import BaseCommand

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Delete all log files"

    def handle(self, *args, **kwargs):
        try:
            logs_path = path.join(settings.BASE_DIR, "logs")
            files = listdir(logs_path)

            for file in files:
                file_path = path.join(logs_path, file)
                try:
                    self.stdout.write(f"Deleting file: {file_path}")
                    remove(file_path)
                except Exception as e:
                    warning_message = (
                        f"Could not delete file {file_path}: {e!s}"
                    )
                    logger.warning(warning_message)
                    self.stdout.write(self.style.WARNING(warning_message))

            message = "Attempted to remove all log files."
            logger.info(message)
            self.stdout.write(self.style.SUCCESS(message))
        except Exception as e:
            error_message = f"Error while deleting log files: {e!s}"
            logger.error(error_message)
            self.stdout.write(self.style.ERROR(error_message))



from unittest.mock import MagicMock, patch

import pytest

from meili.querysets import BoundingBox, IndexQuerySet, Point, Radius


class MockModel:
    __name__ = "MockModel"
    _meilisearch = {"index_name": "test_index", "supports_geo": False}

    class MeiliMeta:
        primary_key = "id"

    objects = MagicMock()


class MockGeoModel:
    __name__ = "MockGeoModel"
    _meilisearch = {"index_name": "test_geo_index", "supports_geo": True}

    class MeiliMeta:
        primary_key = "id"

    objects = MagicMock()


class TestIndexQuerySet:
    def setup_method(self):
        self.mock_index = MagicMock()
        self.mock_client = MagicMock()
        self.mock_client.get_index.return_value = self.mock_index

    @patch("meili.querysets.client")
    def test_queryset_initialization(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)

        assert queryset.model == MockModel
        assert queryset.index == self.mock_index
        mock_client.get_index.assert_called_once_with("test_index")

        assert queryset._IndexQuerySet__offset == 0
        assert queryset._IndexQuerySet__limit == 20
        assert queryset._IndexQuerySet__filters == []
        assert queryset._IndexQuerySet__facets == []
        assert queryset._IndexQuerySet__attributes_to_retrieve == ["*"]
        assert queryset._IndexQuerySet__attributes_to_crop == []
        assert queryset._IndexQuerySet__crop_length == 10
        assert queryset._IndexQuerySet__crop_marker == "..."
        assert queryset._IndexQuerySet__attributes_to_highlight == ["*"]
        assert queryset._IndexQuerySet__highlight_pre_tag == "<mark>"
        assert queryset._IndexQuerySet__highlight_post_tag == "</mark>"
        assert not queryset._IndexQuerySet__show_matches_position
        assert queryset._IndexQuerySet__sort == []
        assert queryset._IndexQuerySet__matching_strategy == "last"
        assert not queryset._IndexQuerySet__show_ranking_score
        assert queryset._IndexQuerySet__attributes_to_search_on == ["*"]
        assert queryset._IndexQuerySet__locales == []

    @patch("meili.querysets.client")
    def test_queryset_repr_and_str(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)

        assert repr(queryset) == "<IndexQuerySet for MockModel>"
        assert str(queryset) == "IndexQuerySet for MockModel"

    @patch("meili.querysets.client")
    def test_queryset_getitem_slice(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset[10:50]

        assert result == queryset
        assert queryset._IndexQuerySet__offset == 10
        assert queryset._IndexQuerySet__limit == 50

    @patch("meili.querysets.client")
    def test_queryset_getitem_invalid_index(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)

        with pytest.raises(
            TypeError, match="IndexQuerySet indices must be slices"
        ):
            queryset[5]

    @patch("meili.querysets.client")
    def test_count(self, mock_client):
        mock_client.get_index.return_value = self.mock_index
        mock_stats = MagicMock()
        mock_stats.number_of_documents = 42
        self.mock_index.get_stats.return_value = mock_stats

        queryset = IndexQuerySet(MockModel)
        count = queryset.count()

        assert count == 42
        self.mock_index.get_stats.assert_called_once()

    @patch("meili.querysets.client")
    def test_paginate(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.paginate(limit=100, offset=25)

        assert result == queryset
        assert queryset._IndexQuerySet__limit == 100
        assert queryset._IndexQuerySet__offset == 25

    @patch("meili.querysets.client")
    def test_order_by_ascending(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.order_by("title", "created_at")

        assert result == queryset
        assert queryset._IndexQuerySet__sort == ["title:asc", "created_at:asc"]

    @patch("meili.querysets.client")
    def test_order_by_descending(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.order_by("-title", "-created_at")

        assert result == queryset
        assert queryset._IndexQuerySet__sort == [
            "title:desc",
            "created_at:desc",
        ]

    @patch("meili.querysets.client")
    def test_order_by_mixed(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.order_by("title", "-created_at", "priority")

        assert result == queryset
        assert queryset._IndexQuerySet__sort == [
            "title:asc",
            "created_at:desc",
            "priority:asc",
        ]

    @patch("meili.querysets.client")
    def test_order_by_geopoint(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.order_by("geoPoint", "-geoPoint")

        assert result == queryset
        assert queryset._IndexQuerySet__sort == [
            "_geoPoint:asc",
            "_geoPoint:desc",
        ]

    @patch("meili.querysets.client")
    def test_matching_strategy(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.matching_strategy("all")

        assert result == queryset
        assert queryset._IndexQuerySet__matching_strategy == "all"

    @patch("meili.querysets.client")
    def test_attributes_to_search_on(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)

        with pytest.raises(
            TypeError, match="list.append\\(\\) takes exactly one argument"
        ):
            queryset.attributes_to_search_on("title", "content")

        result = queryset.attributes_to_search_on("title")
        assert result == queryset

    @patch("meili.querysets.client")
    def test_filter_regular_exact_string(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(title="Test Title")

        assert result == queryset
        assert "title = 'Test Title'" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_regular_exact_number(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(price=100)

        assert result == queryset
        assert "price = 100" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_regular_empty_values(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        queryset.filter(empty_string="", empty_list=[], empty_dict={})

        filters = queryset._IndexQuerySet__filters
        assert "empty_string IS EMPTY" in filters
        assert "empty_list IS EMPTY" in filters
        assert "empty_dict IS EMPTY" in filters

    @patch("meili.querysets.client")
    def test_filter_regular_null_value(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        queryset.filter(nullable_field=None)

        assert "nullable_field IS NULL" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_gte(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(price__gte=100)

        assert result == queryset
        assert "price >= 100" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_gte_invalid_type(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)

        with pytest.raises(
            TypeError, match="Cannot compare <class 'str'> with int or float"
        ):
            queryset.filter(price__gte="invalid")

    @patch("meili.querysets.client")
    def test_filter_gt(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(price__gt=100.5)

        assert result == queryset
        assert "price > 100.5" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_lte(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(price__lte=200)

        assert result == queryset
        assert "price <= 200" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_lt(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(price__lt=50.25)

        assert result == queryset
        assert "price < 50.25" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_in(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(category__in=["electronics", "books"])

        assert result == queryset
        assert (
            "category IN ['electronics', 'books']"
            in queryset._IndexQuerySet__filters
        )

    @patch("meili.querysets.client")
    def test_filter_in_invalid_type(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)

        with pytest.raises(
            TypeError, match="Cannot compare <class 'str'> with list"
        ):
            queryset.filter(category__in="invalid")

    @patch("meili.querysets.client")
    def test_filter_range_list(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(price__range=[10, 100])

        assert result == queryset
        assert "price 10 TO 100" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_range_tuple(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(price__range=(5, 50))

        assert result == queryset
        assert "price 5 TO 50" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_range_range_object(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(price__range=range(1, 10))

        assert result == queryset
        assert "price 1 TO 10" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_range_invalid_type(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)

        with pytest.raises(
            TypeError,
            match="Cannot compare <class 'str'> with range, list or tuple",
        ):
            queryset.filter(price__range="invalid")

    @patch("meili.querysets.client")
    def test_filter_exists_true(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(field__exists=True)

        assert result == queryset
        assert "field EXISTS" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_exists_false(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(field__exists=False)

        assert result == queryset
        assert "field NOT EXISTS" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_exists_invalid_type(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)

        with pytest.raises(
            TypeError, match="Cannot compare <class 'str'> with bool"
        ):
            queryset.filter(field__exists="invalid")

    @patch("meili.querysets.client")
    def test_filter_isnull_true(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(field__isnull=True)

        assert result == queryset
        assert "field IS NULL" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_isnull_false(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        result = queryset.filter(field__isnull=False)

        assert result == queryset
        assert "field NOT IS NULL" in queryset._IndexQuerySet__filters

    @patch("meili.querysets.client")
    def test_filter_isnull_invalid_type(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)

        with pytest.raises(
            TypeError, match="Cannot compare <class 'str'> with bool"
        ):
            queryset.filter(field__isnull="invalid")

    @patch("meili.querysets.client")
    def test_filter_geo_radius(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockGeoModel)
        radius = Radius(lat=48.8566, lng=2.3522, radius=1000)
        result = queryset.filter(radius)

        assert result == queryset
        assert (
            "_geoRadius(48.8566, 2.3522, 1000)"
            in queryset._IndexQuerySet__filters
        )

    @patch("meili.querysets.client")
    def test_filter_geo_bounding_box(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockGeoModel)
        bbox = BoundingBox(top_right=(48.9, 2.4), bottom_left=(48.8, 2.3))
        result = queryset.filter(bbox)

        assert result == queryset
        assert (
            "_geoBoundingBox([48.9, 2.4], [48.8, 2.3])"
            in queryset._IndexQuerySet__filters
        )

    @patch("meili.querysets.client")
    def test_filter_geo_unsupported_model(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockModel)
        radius = Radius(lat=48.8566, lng=2.3522, radius=1000)

        with pytest.raises(
            TypeError, match="Model MockModel does not support geo filters"
        ):
            queryset.filter(radius)

    @patch("meili.querysets.client")
    def test_filter_geo_invalid_type(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        queryset = IndexQuerySet(MockGeoModel)

        with pytest.raises(
            TypeError,
            match="Unnamed Argument must be of type Radius or BoundingBox",
        ):
            queryset.filter("invalid_geo_filter")

    @patch("meili.querysets.client")
    def test_search_basic(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        mock_search_results = {
            "hits": [
                {"id": 1, "title": "Test Item 1"},
                {"id": 2, "title": "Test Item 2"},
            ],
            "estimatedTotalHits": 2,
        }
        self.mock_index.search.return_value = mock_search_results

        mock_obj1 = MagicMock()
        mock_obj1.pk = 1
        mock_obj2 = MagicMock()
        mock_obj2.pk = 2
        MockModel.objects.filter.return_value = [mock_obj1, mock_obj2]

        queryset = IndexQuerySet(MockModel)
        results = queryset.search("test query")

        expected_search_params = {
            "filter": [],
            "facets": [],
            "offset": 0,
            "limit": 20,
            "attributesToRetrieve": ["*"],
            "attributesToCrop": [],
            "cropLength": 10,
            "cropMarker": "...",
            "attributesToHighlight": ["*"],
            "highlightPreTag": "<mark>",
            "highlightPostTag": "</mark>",
            "showMatchesPosition": False,
            "sort": [],
            "matchingStrategy": "last",
            "showRankingScore": False,
            "attributesToSearchOn": ["*"],
            "locales": [],
        }
        self.mock_index.search.assert_called_once_with(
            "test query", expected_search_params
        )

        assert "results" in results
        assert "estimated_total_hits" in results
        assert "offset" in results
        assert "limit" in results
        assert results["estimated_total_hits"] == 2
        assert results["offset"] == 0
        assert results["limit"] == 20
        assert len(results["results"]) == 2

    @patch("meili.querysets.client")
    def test_search_empty_query(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        mock_search_results = {"hits": [], "estimatedTotalHits": 0}
        self.mock_index.search.return_value = mock_search_results
        MockModel.objects.filter.return_value = []

        queryset = IndexQuerySet(MockModel)
        _ = queryset.search()

        self.mock_index.search.assert_called_once()
        call_args = self.mock_index.search.call_args
        assert call_args[0][0] == ""

    @patch("meili.querysets.client")
    def test_search_with_filters_and_options(self, mock_client):
        mock_client.get_index.return_value = self.mock_index

        mock_search_results = {
            "hits": [
                {
                    "id": 1,
                    "_formatted": {"title": "<mark>test</mark>"},
                    "_rankingScore": 0.95,
                }
            ],
            "estimatedTotalHits": 1,
        }
        self.mock_index.search.return_value = mock_search_results

        mock_obj = MagicMock()
        mock_obj.pk = 1
        MockModel.objects.filter.return_value = [mock_obj]

        queryset = IndexQuerySet(MockModel)
        queryset.filter(category="electronics", price__gte=100)
        queryset.order_by("-created_at")
        queryset.paginate(limit=10, offset=5)
        queryset.matching_strategy("all")

        results = queryset.search("test")

        call_args = self.mock_index.search.call_args[0][1]
        assert "category = 'electronics'" in call_args["filter"]
        assert "price >= 100" in call_args["filter"]
        assert call_args["sort"] == ["created_at:desc"]
        assert call_args["limit"] == 10
        assert call_args["offset"] == 5
        assert call_args["matchingStrategy"] == "all"

        enriched_result = results["results"][0]
        assert "object" in enriched_result
        assert "_formatted" in enriched_result
        assert "_rankingScore" in enriched_result


class TestNamedTuples:
    def test_radius_creation(self):
        radius = Radius(lat=48.8566, lng=2.3522, radius=1000)

        assert radius.lat == 48.8566
        assert radius.lng == 2.3522
        assert radius.radius == 1000

    def test_radius_with_string_coords(self):
        radius = Radius(lat="48.8566", lng="2.3522", radius=1000)

        assert radius.lat == "48.8566"
        assert radius.lng == "2.3522"
        assert radius.radius == 1000

    def test_bounding_box_creation(self):
        bbox = BoundingBox(top_right=(48.9, 2.4), bottom_left=(48.8, 2.3))

        assert bbox.top_right == (48.9, 2.4)
        assert bbox.bottom_left == (48.8, 2.3)

    def test_bounding_box_with_string_coords(self):
        bbox = BoundingBox(
            top_right=("48.9", "2.4"), bottom_left=("48.8", "2.3")
        )

        assert bbox.top_right == ("48.9", "2.4")
        assert bbox.bottom_left == ("48.8", "2.3")

    def test_point_creation(self):
        point = Point(lat=48.8566, lng=2.3522)

        assert point.lat == 48.8566
        assert point.lng == 2.3522

    def test_point_with_string_coords(self):
        point = Point(lat="48.8566", lng="2.3522")

        assert point.lat == "48.8566"
        assert point.lng == "2.3522"



from django.contrib.postgres.indexes import BTreeIndex
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext_lazy as _
from django_stubs_ext.db.models import TypedModelMeta
from parler.models import TranslatableModel, TranslatedFields

from core.models import PublishableModel, TimeStampMixinModel, UUIDModel
from product.enum.review import RateEnum, ReviewStatus
from product.managers.review import ProductReviewManager


class ProductReview(
    TranslatableModel, TimeStampMixinModel, PublishableModel, UUIDModel
):
    id = models.BigAutoField(primary_key=True)
    product = models.ForeignKey(
        "product.Product",
        related_name="reviews",
        on_delete=models.CASCADE,
    )
    user = models.ForeignKey(
        "user.UserAccount",
        related_name="product_reviews",
        on_delete=models.CASCADE,
    )
    rate = models.PositiveSmallIntegerField(_("Rate"), choices=RateEnum)
    status = models.CharField(
        _("Status"),
        max_length=250,
        choices=ReviewStatus,
        default=ReviewStatus.NEW,
    )
    translations = TranslatedFields(
        comment=models.TextField(_("Comment"), blank=True, null=True)
    )

    objects: ProductReviewManager = ProductReviewManager()

    class Meta(TypedModelMeta):
        verbose_name = _("Product Review")
        verbose_name_plural = _("Product Reviews")
        ordering = ["-created_at"]
        constraints = [
            models.UniqueConstraint(
                fields=["product", "user"], name="unique_product_review"
            )
        ]
        indexes = [
            *TimeStampMixinModel.Meta.indexes,
            *PublishableModel.Meta.indexes,
            BTreeIndex(fields=["status"], name="product_review_status_ix"),
            BTreeIndex(fields=["rate"], name="product_review_rate_ix"),
            BTreeIndex(fields=["product"], name="product_review_product_ix"),
            BTreeIndex(fields=["user"], name="product_review_user_ix"),
            BTreeIndex(
                fields=["product", "status"],
                name="prod_rev_product_status_ix",
            ),
            BTreeIndex(
                fields=["product", "rate"],
                name="prod_rev_product_rate_ix",
            ),
        ]

    def __str__(self):
        comment_snippet = (
            (
                self.safe_translation_getter("comment", any_language=True)[:50]
                + "..."
            )
            if self.comment
            else "No Comment"
        )
        return (
            f"Review by {self.user.email} on {self.product}: {comment_snippet}"
        )

    def clean(self):
        valid_rates = [choice[0] for choice in RateEnum.choices]
        if self.rate not in valid_rates:
            raise ValidationError(_("Invalid rate value."))
        super().clean()



from decimal import Decimal

from django import forms
from django.utils.translation import gettext_lazy as _


class ApplyDiscountForm(forms.Form):
    discount_percent = forms.DecimalField(
        label=_("Discount Percentage"),
        required=True,
        min_value=Decimal("0"),
        max_value=Decimal("100"),
        decimal_places=2,
        max_digits=5,
        widget=forms.NumberInput(
            attrs={
                "type": "number",
                "placeholder": "0.00",
                "step": "0.01",
                "min": "0",
                "max": "100",
                "class": "border border-gray-300 bg-white font-medium min-w-20 placeholder-gray-400 rounded-lg shadow-sm text-gray-900 text-sm focus:outline-none focus:ring-2 focus:ring-blue-500 focus:border-transparent dark:bg-gray-900 dark:border-gray-600 dark:text-gray-100 dark:placeholder-gray-500 px-3 py-2 w-full",
            }
        ),
        error_messages={
            "required": _("Please enter a discount percentage."),
            "invalid": _("Please enter a valid number."),
            "min_value": _("Discount cannot be negative."),
            "max_value": _("Discount cannot exceed 100%."),
        },
        help_text=_(
            "Enter a value between 0 and 100. For example, 25 for 25% discount."
        ),
    )

    apply_to_inactive = forms.BooleanField(
        label=_("Apply to inactive products"),
        required=False,
        initial=False,
        widget=forms.CheckboxInput(
            attrs={
                "class": "rounded border-gray-300 text-primary-600 focus:ring-primary-500 dark:border-gray-600 dark:bg-gray-700"
            }
        ),
        help_text=_("Check this to also apply discount to inactive products"),
    )

    def clean_discount_percent(self):
        value = self.cleaned_data.get("discount_percent")
        if value is not None:
            return Decimal(str(value))
        return value



# Generated by Django 5.0.6 on 2024-05-16 11:58
import django.contrib.auth.validators
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0008_remove_useraccount_user_account_trgm_idx_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="useraccount",
            name="username",
            field=models.CharField(
                blank=True,
                error_messages={"unique": "A user with that username already exists."},
                help_text="Required. 30 characters or fewer.Letters, digits and @/./+/-/_ only.",
                max_length=30,
                null=True,
                unique=True,
                validators=[django.contrib.auth.validators.UnicodeUsernameValidator()],
                verbose_name="Username",
            ),
        ),
    ]



import logging

from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import extend_schema
from rest_framework import generics

from contact.models import Contact
from contact.serializers import ContactWriteSerializer
from core.api.serializers import ErrorResponseSerializer

logger = logging.getLogger(__name__)


class ContactCreateView(generics.CreateAPIView):
    queryset = Contact.objects.all()
    serializer_class = ContactWriteSerializer

    @extend_schema(
        operation_id="createContact",
        summary=_("Create a contact message"),
        description=_("Send a contact message to the site administrators."),
        tags=["Contact"],
        responses={
            201: ContactWriteSerializer,
            400: ErrorResponseSerializer,
        },
    )
    def post(self, request, *args, **kwargs):
        return super().post(request, *args, **kwargs)



from django.contrib.auth import get_user_model
from django.db import models
from django.utils.translation import gettext_lazy as _
from drf_spectacular.helpers import lazy_serializer
from drf_spectacular.utils import extend_schema_field
from parler_rest.serializers import TranslatableModelSerializer
from rest_framework import serializers
from rest_framework.relations import PrimaryKeyRelatedField

from user.serializers.account import UserDetailsSerializer
from blog.models.author import BlogAuthor
from core.api.schema import generate_schema_multi_lang
from core.utils.serializers import TranslatedFieldExtended

User = get_user_model()


@extend_schema_field(generate_schema_multi_lang(BlogAuthor))
class TranslatedFieldsFieldExtend(TranslatedFieldExtended):
    pass


class BlogAuthorSerializer(
    TranslatableModelSerializer,
    serializers.ModelSerializer[BlogAuthor],
):
    user = PrimaryKeyRelatedField(queryset=User.objects.all())
    translations = TranslatedFieldsFieldExtend(shared_model=BlogAuthor)
    website = serializers.SerializerMethodField()

    @extend_schema_field(
        {
            "type": "string",
            "nullable": True,
            "maxLength": 200,
            "description": _("URL link or empty string"),
        }
    )
    def get_website(self, obj) -> str | None:
        return obj.website

    class Meta:
        model = BlogAuthor
        fields = (
            "translations",
            "id",
            "uuid",
            "user",
            "website",
            "created_at",
            "updated_at",
            "number_of_posts",
            "total_likes_received",
        )
        read_only_fields = (
            "created_at",
            "updated_at",
            "uuid",
            "number_of_posts",
            "total_likes_received",
        )


class BlogAuthorDetailSerializer(BlogAuthorSerializer):
    user = UserDetailsSerializer(read_only=True)
    recent_posts = serializers.SerializerMethodField()
    top_posts = serializers.SerializerMethodField()

    class Meta(BlogAuthorSerializer.Meta):
        fields = (
            *BlogAuthorSerializer.Meta.fields,
            "recent_posts",
            "top_posts",
        )

    @extend_schema_field(
        lazy_serializer("blog.serializers.post.BlogPostSerializer")(many=True)
    )
    def get_recent_posts(self, obj: BlogAuthor):
        from blog.serializers.post import BlogPostSerializer  # noqa: PLC0415, I001

        recent_posts = obj.blog_posts.order_by("-created_at")[:3]
        return BlogPostSerializer(
            recent_posts, many=True, context=self.context
        ).data

    @extend_schema_field(
        lazy_serializer("blog.serializers.post.BlogPostSerializer")(many=True)
    )
    def get_top_posts(self, obj: BlogAuthor):
        from blog.serializers.post import BlogPostSerializer  # noqa: PLC0415, I001

        top_posts = obj.blog_posts.annotate(
            likes_count_field=models.Count("likes")
        ).order_by("-view_count", "-likes_count_field")[:3]
        return BlogPostSerializer(
            top_posts, many=True, context=self.context
        ).data


class BlogAuthorWriteSerializer(
    TranslatableModelSerializer,
    serializers.ModelSerializer[BlogAuthor],
):
    user = PrimaryKeyRelatedField(queryset=User.objects.all())
    translations = TranslatedFieldsFieldExtend(shared_model=BlogAuthor)

    class Meta:
        model = BlogAuthor
        fields = (
            "translations",
            "user",
            "website",
        )

    def validate_user(self, value: User) -> User:
        if (
            self.instance is None
            and BlogAuthor.objects.filter(user=value).exists()
        ):
            raise serializers.ValidationError(
                _("This user already has an author profile.")
            )
        return value

    def validate_website(self, value: str) -> str:
        if value and not (
            value.startswith("http://") or value.startswith("https://")
        ):
            raise serializers.ValidationError(
                _("Website must start with http:// or https://")
            )
        return value



# Generated by Django 5.2.6 on 2025-09-25 21:34

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blog', '0029_alter_blogcomment_approved'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddIndex(
            model_name='blogcategory',
            index=models.Index(fields=['tree_id', 'lft'], name='blog_blogcategory_tree_id_9462'),
        ),
        migrations.AddIndex(
            model_name='blogcomment',
            index=models.Index(fields=['tree_id', 'lft'], name='blog_blogcomment_tree_id_ld9de'),
        ),
    ]



# Generated by Django 5.1.4 on 2024-12-31 00:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notification', '0006_notificationuser_notificatio_user_id_f8507b_idx_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='link',
            field=models.URLField(blank=True, default='', verbose_name='Link'),
        ),
    ]



from datetime import timedelta
from decimal import Decimal

import admin_thumbnails
from django.contrib import admin, messages
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse_lazy
from django.utils import timezone
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext
from mptt.admin import DraggableMPTTAdmin
from parler.admin import TranslatableAdmin
from simple_history.admin import SimpleHistoryAdmin
from unfold.admin import ModelAdmin, TabularInline
from unfold.contrib.filters.admin import (
    DropdownFilter,
    RangeDateTimeFilter,
    RangeNumericFilter,
    RangeNumericListFilter,
    RelatedDropdownFilter,
    SliderNumericFilter,
)
from unfold.decorators import action
from unfold.enums import ActionVariant

from core.admin import ExportModelAdmin
from core.forms.measurement import MeasurementWidget
from core.units import WeightUnits
from product.enum.review import ReviewStatus
from product.forms import ApplyDiscountForm
from product.models.category import ProductCategory
from product.models.category_image import ProductCategoryImage
from product.models.favourite import ProductFavourite
from product.models.image import ProductImage
from product.models.product import Product
from product.models.review import ProductReview
from tag.admin import TaggedItemInline


class StockStatusFilter(DropdownFilter):
    title = _("Stock Status")
    parameter_name = "stock_status"

    def lookups(self, request, model_admin):
        return [
            ("in_stock", _("In Stock (>0)")),
            ("low_stock", _("Low Stock (1-10)")),
            ("out_of_stock", _("Out of Stock (0)")),
            ("high_stock", _("High Stock (>50)")),
            ("critical_stock", _("Critical Stock (1-5)")),
        ]

    def queryset(self, request, queryset):
        filter_value = self.value()

        match filter_value:
            case "in_stock":
                filter_kwargs = {"stock__gt": 0}
            case "low_stock":
                filter_kwargs = {"stock__range": (1, 10)}
            case "out_of_stock":
                filter_kwargs = {"stock": 0}
            case "high_stock":
                filter_kwargs = {"stock__gt": 50}
            case "critical_stock":
                filter_kwargs = {"stock__range": (1, 5)}
            case _:
                return queryset

        return queryset.filter(**filter_kwargs)


class PriceRangeFilter(DropdownFilter):
    title = _("Price Range")
    parameter_name = "price_range"

    def lookups(self, request, model_admin):
        return [
            ("budget", _("Budget (€0-€20)")),
            ("affordable", _("Affordable (€20-€50)")),
            ("mid_range", _("Mid-range (€50-€100)")),
            ("premium", _("Premium (€100-€500)")),
            ("luxury", _("Luxury (€500+)")),
        ]

    def queryset(self, request, queryset):
        filter_value = self.value()

        match filter_value:
            case "budget":
                filter_kwargs = {"price__lte": 20}
            case "affordable":
                filter_kwargs = {"price__range": (20, 50)}
            case "mid_range":
                filter_kwargs = {"price__range": (50, 100)}
            case "premium":
                filter_kwargs = {"price__range": (100, 500)}
            case "luxury":
                filter_kwargs = {"price__gt": 500}
            case _:
                return queryset

        return queryset.filter(**filter_kwargs)


class DiscountStatusFilter(DropdownFilter):
    title = _("Discount Status")
    parameter_name = "discount_status"

    def lookups(self, request, model_admin):
        return [
            ("on_sale", _("On Sale (>0%)")),
            ("heavy_discount", _("Heavy Discount (>20%)")),
            ("clearance", _("Clearance (>50%)")),
            ("no_discount", _("No Discount (0%)")),
        ]

    def queryset(self, request, queryset):
        filter_value = self.value()

        match filter_value:
            case "on_sale":
                filter_kwargs = {"discount_percent__gt": 0}
            case "heavy_discount":
                filter_kwargs = {"discount_percent__gt": 20}
            case "clearance":
                filter_kwargs = {"discount_percent__gt": 50}
            case "no_discount":
                filter_kwargs = {"discount_percent": 0}
            case _:
                return queryset

        return queryset.filter(**filter_kwargs)


class PopularityFilter(DropdownFilter):
    title = _("Popularity")
    parameter_name = "popularity"

    def lookups(self, request, model_admin):
        return [
            ("trending", _("Trending (High Views)")),
            ("loved", _("Loved (High Likes)")),
            ("well_reviewed", _("Well Reviewed (>4.0)")),
            ("new_arrivals", _("New Arrivals (Last 30 days)")),
        ]

    def queryset(self, request, queryset):
        filter_value = self.value()

        match filter_value:
            case "trending":
                filter_kwargs = {"view_count__gt": 100}
            case "loved":
                queryset = queryset.with_likes_count_annotation()
                filter_kwargs = {"likes_count_annotation__gt": 10}
            case "well_reviewed":
                queryset = queryset.with_review_average_annotation()
                filter_kwargs = {"review_average_annotation__gt": 7.0}
            case "new_arrivals":
                thirty_days_ago = timezone.now() - timedelta(days=30)
                filter_kwargs = {"created_at__gte": thirty_days_ago}
            case _:
                return queryset

        return queryset.filter(**filter_kwargs)


class LikesCountFilter(RangeNumericListFilter):
    title = _("Likes count")
    parameter_name = "likes_count"

    def queryset(self, request, queryset):
        filters = {}

        queryset = queryset.with_likes_count_annotation()

        value_from = self.used_parameters.get(f"{self.parameter_name}_from")
        if value_from and value_from != "":
            filters["likes_count_annotation__gte"] = value_from

        value_to = self.used_parameters.get(f"{self.parameter_name}_to")
        if value_to and value_to != "":
            filters["likes_count_annotation__lte"] = value_to

        return queryset.filter(**filters) if filters else queryset

    def expected_parameters(self):
        return [
            f"{self.parameter_name}_from",
            f"{self.parameter_name}_to",
        ]


class ReviewAverageFilter(RangeNumericListFilter):
    title = _("Review Rating")
    parameter_name = "review_average"

    def queryset(self, request, queryset):
        filters = {}

        queryset = queryset.with_review_average_annotation()

        value_from = self.used_parameters.get(f"{self.parameter_name}_from")
        if value_from and value_from != "":
            filters["review_average_annotation__gte"] = value_from

        value_to = self.used_parameters.get(f"{self.parameter_name}_to")
        if value_to and value_to != "":
            filters["review_average_annotation__lte"] = value_to

        return queryset.filter(**filters) if filters else queryset

    def expected_parameters(self):
        return [
            f"{self.parameter_name}_from",
            f"{self.parameter_name}_to",
        ]


@admin_thumbnails.thumbnail("image")
class ProductImageInline(TabularInline):
    model = ProductImage
    extra = 0
    fields = ("image_preview", "image", "is_main")
    readonly_fields = ("image_preview",)

    tab = True
    show_change_link = True

    def image_preview(self, obj):
        if obj.image:
            return format_html(
                '<img src="{}" style="width: 60px; height: 60px; object-fit: cover; border-radius: 8px; border: 1px solid #e5e7eb;" />',
                obj.image.url,
            )
        return format_html(
            '<div class="bg-gray-100 dark:bg-gray-900 rounded border border-gray-200 dark:border-gray-700 flex items-center justify-center text-base-400 dark:text-base-500 text-xs">No Image</div>'
        )

    image_preview.short_description = _("Preview")


@admin.register(Product)
class ProductAdmin(
    TranslatableAdmin, ExportModelAdmin, SimpleHistoryAdmin, ModelAdmin
):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = True
    list_filter_submit = True
    list_filter_sheet = True

    list_display = [
        "product_info",
        "category_display",
        "pricing_info",
        "stock_info",
        "performance_metrics",
        "status_badges",
        "created_display",
    ]
    search_fields = [
        "id",
        "sku",
        "translations__name",
        "translations__description",
        "category__translations__name",
    ]
    list_filter = [
        StockStatusFilter,
        PriceRangeFilter,
        DiscountStatusFilter,
        PopularityFilter,
        "active",
        ("category", RelatedDropdownFilter),
        ("vat", RelatedDropdownFilter),
        ("stock", RangeNumericFilter),
        ("price", RangeNumericFilter),
        ("discount_percent", SliderNumericFilter),
        ("view_count", RangeNumericFilter),
        ("created_at", RangeDateTimeFilter),
        ("updated_at", RangeDateTimeFilter),
        LikesCountFilter,
        ReviewAverageFilter,
    ]
    inlines = [ProductImageInline, TaggedItemInline]
    readonly_fields = (
        "id",
        "uuid",
        "sku",
        "created_at",
        "updated_at",
        "view_count",
        "likes_count",
        "product_analytics",
        "pricing_summary",
        "performance_summary",
    )
    list_select_related = ["category", "vat", "changed_by"]
    list_per_page = 25
    actions = [
        "make_active",
        "make_inactive",
        "apply_custom_discount",
        "clear_discount",
    ]
    date_hierarchy = "created_at"

    fieldsets = (
        (
            _("Product Information"),
            {
                "fields": (
                    "sku",
                    "category",
                    "active",
                    "name",
                    "description",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("Pricing"),
            {
                "fields": (
                    "price",
                    "discount_percent",
                    "vat",
                    "pricing_summary",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("Inventory"),
            {
                "fields": (
                    "stock",
                    "weight",
                ),
                "classes": ("wide",),
            },
        ),
        (
            _("SEO & Marketing"),
            {
                "fields": (
                    "slug",
                    "seo_title",
                    "seo_description",
                    "seo_keywords",
                ),
                "classes": ("collapse",),
            },
        ),
        (
            _("Performance Metrics"),
            {
                "fields": (
                    "view_count",
                    "likes_count",
                    "performance_summary",
                ),
                "classes": ("collapse",),
            },
        ),
        (
            _("Analytics"),
            {
                "fields": ("product_analytics",),
                "classes": ("collapse",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": (
                    "id",
                    "uuid",
                    "created_at",
                    "updated_at",
                    "changed_by",
                ),
                "classes": ("collapse",),
            },
        ),
    )

    def get_form(self, request, obj=None, change=False, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        form.base_fields["weight"].widget = MeasurementWidget(
            unit_choices=WeightUnits.CHOICES
        )
        return form

    def get_queryset(self, request):
        return (
            super()
            .get_queryset(request)
            .with_likes_count_annotation()
            .with_review_average_annotation()
            .select_related("category", "vat", "changed_by")
            .prefetch_related("images", "reviews", "favourited_by")
        )

    def get_prepopulated_fields(self, request, obj=None):
        return {"slug": ("name",)}

    def product_info(self, obj):
        name = (
            obj.safe_translation_getter("name", any_language=True)
            or "Untitled Product"
        )
        main_image = obj.images.filter(is_main=True).first()

        image_html = ""
        if main_image and main_image.image:
            image_html = format_html(
                '<img src="{}" style="width: 50px; height: 50px; object-fit: cover; border-radius: 8px; border: 1px solid #e5e7eb; margin-right: 12px;" />',
                main_image.image.url,
            )
        else:
            image_html = format_html(
                '<div style="width: 50px; height: 50px; background: #f3f4f6; border-radius: 8px; border: 1px solid #e5e7eb; margin-right: 12px; display: flex; align-items: center; justify-content: center; color: #9ca3af; font-size: 20px;">📦</div>'
            )

        return format_html(
            '<div style="display: flex; align-items: center;">'
            "{}"
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="text-base-500 dark:text-base-400">#{}</div>'
            "</div>"
            "</div>",
            image_html,
            name,
            obj.sku[:8],
        )

    product_info.short_description = _("Product")

    def category_display(self, obj):
        if obj.category:
            category_name = (
                obj.category.safe_translation_getter("name", any_language=True)
                or "Unnamed"
            )
            category_path = " → ".join(
                [
                    cat.safe_translation_getter("name", any_language=True)
                    or "Unnamed"
                    for cat in obj.category.get_ancestors()
                ]
            )

            return format_html(
                '<div class="text-sm">'
                '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
                '<div class="text-xs text-base-500 dark:text-base-400">{}</div>'
                "</div>",
                category_name,
                category_path if category_path else "Root",
            )
        return format_html(
            '<span class="text-base-400 dark:text-base-500">No Category</span>'
        )

    category_display.short_description = _("Category")

    def pricing_info(self, obj):
        price = obj.price
        final_price = obj.final_price
        discount = obj.discount_percent

        price_class = (
            "text-base-500 dark:text-base-400 line-through"
            if discount > 0
            else "font-bold text-base-900 dark:text-base-100"
        )

        if discount > 0:
            return format_html(
                '<div class="text-sm">'
                '<div class="{}">{}</div>'
                '<div class="font-bold text-green-600 dark:text-green-400">{}</div>'
                '<div class="text-xs text-red-600 dark:text-red-400">-{}%</div>'
                "</div>",
                price_class,
                price,
                final_price,
                discount,
            )
        else:
            return format_html(
                '<div class="text-sm"><div class="{}">{}</div></div>',
                price_class,
                price,
            )

    pricing_info.short_description = _("Pricing")

    def stock_info(self, obj):
        stock = obj.stock

        if stock == 0:
            stock_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full">'
                "❌ Out of Stock"
                "</span>"
            )
        elif stock <= 5:
            stock_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-orange-50 dark:bg-orange-900 text-orange-700 dark:text-orange-300 rounded-full">'
                "⚠️ Critical ({})"
                "</span>",
                stock,
            )
        elif stock <= 10:
            stock_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-300 rounded-full">'
                "🔶 Low ({})"
                "</span>",
                stock,
            )
        else:
            stock_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full">'
                "✅ In Stock ({})"
                "</span>",
                stock,
            )

        return stock_badge

    stock_info.short_description = _("Stock")

    def performance_metrics(self, obj):
        views = obj.view_count
        likes = obj.likes_count
        rating = obj.review_average
        rating_formatted = "{:.1f}".format(float(rating))

        return format_html(
            '<div class="text-sm">'
            '<div class="text-base-600 dark:text-base-400">👁️ {}</div>'
            '<div class="text-base-600 dark:text-base-400">❤️ {}</div>'
            '<div class="text-base-600 dark:text-base-400">⭐ {}/10</div>'
            "</div>",
            views,
            likes,
            rating_formatted,
        )

    performance_metrics.short_description = _("Performance")

    def status_badges(self, obj):
        badges = []

        if obj.active:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full border border-green-200 dark:border-green-700" title="Product is active and visible to customers">'
                "✅ Active"
                "</span>"
            )
        else:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full border border-red-200 dark:border-red-700" title="Product is inactive and hidden from customers">'
                "❌ Inactive"
                "</span>"
            )

        if obj.stock <= 0:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-gray-50 dark:bg-gray-900 text-base-700 dark:text-base-300 rounded-full border border-gray-200 dark:border-gray-700" title="Product is out of stock">'
                "📦 Out of Stock"
                "</span>"
            )
        elif obj.stock <= 10:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-300 rounded-full border border-yellow-200 dark:border-yellow-700" title="Low stock - only {} units remaining">'
                "⚠️ Low Stock"
                "</span>".format(obj.stock)
            )

        if obj.discount_percent > 0:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-orange-50 dark:bg-orange-900 text-orange-700 dark:text-orange-300 rounded-full border border-orange-200 dark:border-orange-700" title="Product has {}% discount applied">'
                "🏷️ {}% OFF"
                "</span>".format(obj.discount_percent, obj.discount_percent)
            )

        thirty_days_ago = timezone.now() - timedelta(days=30)
        if obj.created_at >= thirty_days_ago:
            days_old = (timezone.now() - obj.created_at).days
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-300 rounded-full border border-blue-200 dark:border-blue-700" title="Product added {} days ago">'
                "🆕 New"
                "</span>".format(days_old)
            )

        if hasattr(obj, "is_featured") and obj.is_featured:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-purple-50 dark:bg-purple-900 text-purple-700 dark:text-purple-300 rounded-full border border-purple-200 dark:border-purple-700" title="This product is featured on the homepage">'
                "⭐ Featured"
                "</span>"
            )

        views = obj.view_count
        if views > 100:
            badges.append(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-indigo-50 dark:bg-indigo-900 text-indigo-700 dark:text-indigo-300 rounded-full border border-indigo-200 dark:border-indigo-700" title="Product has {} views - performing well">'
                "🔥 Popular"
                "</span>".format(views)
            )

        return mark_safe(
            '<div class="flex flex-wrap gap-1">' + "".join(badges) + "</div>"
        )

    status_badges.short_description = _("Status")

    def created_display(self, obj):
        now = timezone.now()
        diff = now - obj.created_at

        if diff < timedelta(days=1):
            time_ago = "Today"
            color = "text-green-600 dark:text-green-400"
        elif diff < timedelta(days=7):
            time_ago = f"{diff.days}d ago"
            color = "text-blue-600 dark:text-blue-400"
        else:
            time_ago = obj.created_at.strftime("%Y-%m-%d")
            color = "text-base-600 dark:text-base-400"

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="{}">{}</div>'
            "</div>",
            obj.created_at.strftime("%Y-%m-%d"),
            color,
            time_ago,
        )

    created_display.short_description = _("Created")

    def pricing_summary(self, obj):
        price = obj.price
        discount = obj.discount_percent
        final_price = obj.final_price
        discount_value = obj.discount_value
        vat_value = obj.vat_value
        vat_percent = obj.vat_percent

        return format_html(
            '<div class="text-sm">'
            '<div class="grid grid-cols-2 gap-2">'
            "<div><strong>Base Price:</strong></div><div>{}</div>"
            "<div><strong>Discount %:</strong></div><div>{}%</div>"
            "<div><strong>Discount Value:</strong></div><div>{}</div>"
            "<div><strong>VAT %:</strong></div><div>{}%</div>"
            "<div><strong>VAT Value:</strong></div><div>{}</div>"
            '<div><strong>Final Price:</strong></div><div class="font-bold">{}</div>'
            "</div>"
            "</div>",
            price,
            discount,
            discount_value,
            vat_percent,
            vat_value,
            final_price,
        )

    pricing_summary.short_description = _("Pricing Summary")

    def performance_summary(self, obj):
        views = obj.view_count
        likes = obj.likes_count
        rating = obj.review_average
        review_count = obj.review_count
        favorites_count = obj.favourited_by.count()

        rating_formatted = "{:.1f}".format(float(rating))
        engagement_formatted = "%.1f" % (
            ((likes + favorites_count) / max(views, 1)) * 100
        )

        return format_html(
            '<div class="text-sm">'
            '<div class="grid grid-cols-2 gap-2">'
            "<div><strong>Total Views:</strong></div><div>{}</div>"
            "<div><strong>Likes:</strong></div><div>{}</div>"
            "<div><strong>Favorites:</strong></div><div>{}</div>"
            "<div><strong>Reviews:</strong></div><div>{}</div>"
            "<div><strong>Avg Rating:</strong></div><div>{}/10</div>"
            "<div><strong>Engagement:</strong></div><div>{}%%</div>"
            "</div>"
            "</div>",
            views,
            likes,
            favorites_count,
            review_count,
            rating_formatted,
            engagement_formatted,
        )

    performance_summary.short_description = _("Performance Summary")

    def product_analytics(self, obj):
        now = timezone.now()
        age = now - obj.created_at
        last_updated = now - obj.updated_at

        views = obj.view_count
        likes = obj.likes_count
        engagement_rate = (likes / max(views, 1)) * 100
        engagement_formatted = "{:.1f}".format(engagement_rate)

        return format_html(
            '<div class="text-sm">'
            '<div class="grid grid-cols-2 gap-2">'
            "<div><strong>Product Age:</strong></div><div>{}d</div>"
            "<div><strong>Last Updated:</strong></div><div>{}d ago</div>"
            "<div><strong>Stock Value:</strong></div><div>{}</div>"
            "<div><strong>Engagement Rate:</strong></div><div>{}%%</div>"
            "<div><strong>Has Images:</strong></div><div>{}</div>"
            "<div><strong>Has Reviews:</strong></div><div>{}</div>"
            "<div><strong>In Category:</strong></div><div>{}</div>"
            "<div><strong>Changed By:</strong></div><div>{}</div>"
            "</div>"
            "</div>",
            age.days,
            last_updated.days,
            obj.final_price.amount * obj.stock,
            engagement_formatted,
            "Yes" if obj.images.exists() else "No",
            "Yes" if obj.reviews.exists() else "No",
            "Yes" if obj.category else "No",
            obj.changed_by.email if obj.changed_by else "System",
        )

    product_analytics.short_description = _("Product Analytics")

    @action(
        description=_("Activate selected products"),
        variant=ActionVariant.SUCCESS,
        icon="check_circle",
    )
    def make_active(self, request, queryset):
        updated = queryset.update(active=True)
        self.message_user(
            request,
            ngettext(
                _("%(count)d product was successfully activated."),
                _("%(count)d products were successfully activated."),
                updated,
            )
            % {"count": updated},
            messages.SUCCESS,
        )

    @action(
        description=_("Deactivate selected products"),
        variant=ActionVariant.WARNING,
        icon="cancel",
    )
    def make_inactive(self, request, queryset):
        updated = queryset.update(active=False)
        self.message_user(
            request,
            ngettext(
                _("%(count)d product was successfully deactivated."),
                _("%(count)d products were successfully deactivated."),
                updated,
            )
            % {"count": updated},
            messages.SUCCESS,
        )

    @action(
        description=_("Apply custom discount to selected products"),
        variant=ActionVariant.INFO,
        icon="local_offer",
        permissions=["change"],
    )
    def apply_custom_discount(self, request, queryset):
        is_action_post = "_selected_action" in request.POST
        is_form_post = "discount_percent" in request.POST

        if is_action_post and not is_form_post:
            selected_ids = list(queryset.values_list("id", flat=True))
            request.session["selected_product_ids"] = selected_ids

            total_count = queryset.count()
            active_count = queryset.filter(active=True).count()
            inactive_count = total_count - active_count

            form = ApplyDiscountForm()

            context = {
                **self.admin_site.each_context(request),
                "title": _("Apply Custom Discount"),
                "form": form,
                "queryset": queryset,
                "total_count": total_count,
                "active_count": active_count,
                "inactive_count": inactive_count,
                "opts": self.model._meta,
                "has_view_permission": self.has_view_permission(request),
                "has_change_permission": self.has_change_permission(request),
                "breadcrumbs_items": [
                    {"title": _("Home"), "link": "admin:index"},
                    {
                        "title": self.model._meta.verbose_name_plural.title(),
                        "link": "admin:product_product_changelist",
                    },
                    {"title": _("Apply Custom Discount")},
                ],
            }

            return render(request, "admin/product/apply_discount.html", context)

        if is_form_post:
            selected_ids = request.session.get("selected_product_ids", [])

            if selected_ids:
                queryset = Product.objects.filter(id__in=selected_ids)
            else:
                messages.error(
                    request, _("No products selected. Please try again.")
                )
                return HttpResponseRedirect(
                    reverse_lazy("admin:product_product_changelist")
                )

            form = ApplyDiscountForm(request.POST)

            if form.is_valid():
                discount_percent = form.cleaned_data["discount_percent"]
                apply_to_inactive = form.cleaned_data["apply_to_inactive"]

                if not apply_to_inactive:
                    queryset = queryset.filter(active=True)

                updated = queryset.update(discount_percent=discount_percent)

                if "selected_product_ids" in request.session:
                    del request.session["selected_product_ids"]

                self.message_user(
                    request,
                    ngettext(
                        _(
                            "Applied %(discount)s%% discount to %(count)d product."
                        ),
                        _(
                            "Applied %(discount)s%% discount to %(count)d products."
                        ),
                        updated,
                    )
                    % {"count": updated, "discount": discount_percent},
                    messages.SUCCESS,
                )

                return HttpResponseRedirect(
                    reverse_lazy("admin:product_product_changelist")
                )
            else:
                total_count = queryset.count()
                active_count = queryset.filter(active=True).count()
                inactive_count = total_count - active_count

                context = {
                    **self.admin_site.each_context(request),
                    "title": _("Apply Custom Discount"),
                    "form": form,
                    "queryset": queryset,
                    "total_count": total_count,
                    "active_count": active_count,
                    "inactive_count": inactive_count,
                    "opts": self.model._meta,
                    "has_view_permission": self.has_view_permission(request),
                    "has_change_permission": self.has_change_permission(
                        request
                    ),
                    "breadcrumbs_items": [
                        {"title": _("Home"), "link": "admin:index"},
                        {
                            "title": self.model._meta.verbose_name_plural.title(),
                            "link": "admin:product_product_changelist",
                        },
                        {"title": _("Apply Custom Discount")},
                    ],
                }

                return render(
                    request, "admin/product/apply_discount.html", context
                )

        return HttpResponseRedirect(
            reverse_lazy("admin:product_product_changelist")
        )

    @action(
        description=_("Clear discount from selected products"),
        variant=ActionVariant.DANGER,
        icon="cancel",
    )
    def clear_discount(self, request, queryset):
        updated = queryset.update(discount_percent=Decimal("0.0"))
        self.message_user(
            request,
            ngettext(
                _("%(count)d product's discount was cleared."),
                _("%(count)d products' discounts were cleared."),
                updated,
            )
            % {"count": updated},
            messages.SUCCESS,
        )


@admin_thumbnails.thumbnail("image")
class ProductCategoryImageInline(TabularInline):
    model = ProductCategoryImage
    extra = 0
    fields = ("image_preview", "image", "image_type", "active")
    readonly_fields = ("image_preview",)

    tab = True
    show_change_link = True

    def image_preview(self, obj):
        if obj.image:
            return format_html(
                '<img src="{}" style="width: 60px; height: 60px; object-fit: cover; border-radius: 8px; border: 1px solid #e5e7eb;" />',
                obj.image.url,
            )
        return format_html(
            '<div class="bg-gray-100 dark:bg-gray-900 rounded border border-gray-200 dark:border-gray-700 flex items-center justify-center text-base-400 dark:text-base-500 text-xs">No Image</div>'
        )

    image_preview.short_description = _("Preview")


@admin.register(ProductCategory)
class ProductCategoryAdmin(ModelAdmin, TranslatableAdmin, DraggableMPTTAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = True
    list_filter_submit = True
    list_filter_sheet = True

    mptt_indent_field = "translations__name"
    list_per_page = 25
    list_display = (
        "tree_actions",
        "indented_title",
        "category_info",
        "category_stats",
        "category_status",
        "image_preview",
        "created_display",
        "products_count_display",
        "recursive_products_display",
    )
    list_display_links = ("category_info",)
    search_fields = ("translations__name", "translations__description")
    list_filter = [
        "active",
        ("created_at", RangeDateTimeFilter),
        ("updated_at", RangeDateTimeFilter),
        "parent",
    ]
    inlines = [ProductCategoryImageInline]
    readonly_fields = [
        "uuid",
        "created_at",
        "updated_at",
        "category_analytics",
        "products_count_display",
        "recursive_products_display",
    ]

    fieldsets = (
        (
            _("Category Information"),
            {
                "fields": ("parent", "slug", "active"),
                "classes": ("wide",),
            },
        ),
        (
            _("Content"),
            {
                "fields": ("name", "description"),
                "classes": ("wide",),
            },
        ),
        (
            _("SEO"),
            {
                "fields": ("seo_title", "seo_description", "seo_keywords"),
                "classes": ("collapse",),
            },
        ),
        (
            _("Analytics"),
            {
                "fields": (
                    "category_analytics",
                    "products_count_display",
                    "recursive_products_display",
                ),
                "classes": ("collapse",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("uuid", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        qs = ProductCategory.objects.add_related_count(
            qs,
            Product,
            "category",
            "products_cumulative_count",
            cumulative=True,
        )
        qs = ProductCategory.objects.add_related_count(
            qs, Product, "category", "products_count", cumulative=False
        )
        return qs

    def get_prepopulated_fields(self, request, obj=None):
        return {"slug": ("name",)}

    def category_info(self, instance):
        name = (
            instance.safe_translation_getter("name", any_language=True)
            or "Unnamed Category"
        )
        level = instance.level
        path = " → ".join(
            [
                cat.safe_translation_getter("name", any_language=True)
                or "Unnamed"
                for cat in instance.get_ancestors()
            ]
        )

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="text-base-500 dark:text-base-400">Level: {}</div>'
            '<div class="text-xs text-base-400 dark:text-base-500">{}</div>'
            "</div>",
            name,
            level,
            path if path else "Root Category",
        )

    category_info.short_description = _("Category")

    def category_stats(self, instance):
        direct_count = getattr(instance, "products_count", 0)
        total_count = getattr(instance, "products_cumulative_count", 0)
        children_count = instance.get_children().count()

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">Direct: {}</div>'
            '<div class="text-base-600 dark:text-base-400">Total: {}</div>'
            '<div class="text-base-500 dark:text-base-500">Subcategories: {}</div>'
            "</div>",
            direct_count,
            total_count,
            children_count,
        )

    category_stats.short_description = _("Product Stats")

    def category_status(self, instance):
        if instance.active:
            status_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full">'
                "✅ Active"
                "</span>"
            )
        else:
            status_badge = format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full">'
                "❌ Inactive"
                "</span>"
            )

        return status_badge

    category_status.short_description = _("Status")

    def image_preview(self, instance):
        main_image = instance.main_image
        if main_image and main_image.image:
            return format_html(
                '<img src="{}" style="width: 40px; height: 40px; object-fit: cover; border-radius: 6px; border: 1px solid #e5e7eb;" />',
                main_image.image.url,
            )
        return format_html(
            '<div class="bg-gray-100 dark:bg-gray-900 rounded border border-gray-200 dark:border-gray-700 flex items-center justify-center text-base-400 dark:text-base-500 text-xs">'
            "📁"
            "</div>"
        )

    image_preview.short_description = _("Image")

    def created_display(self, instance):
        return format_html(
            '<div class="text-sm text-base-600 dark:text-base-400">{}</div>',
            instance.created_at.strftime("%Y-%m-%d"),
        )

    created_display.short_description = _("Created")

    def category_analytics(self, instance):
        ancestors_count = instance.get_ancestors().count()
        descendants_count = instance.get_descendants().count()
        siblings_count = instance.get_siblings().count()

        return format_html(
            '<div class="text-sm">'
            '<div class="grid grid-cols-2 gap-2">'
            "<div><strong>Level:</strong></div><div>{}</div>"
            "<div><strong>Ancestors:</strong></div><div>{}</div>"
            "<div><strong>Descendants:</strong></div><div>{}</div>"
            "<div><strong>Siblings:</strong></div><div>{}</div>"
            "<div><strong>Direct Products:</strong></div><div>{}</div>"
            "<div><strong>Total Products:</strong></div><div>{}</div>"
            "</div>"
            "</div>",
            instance.level,
            ancestors_count,
            descendants_count,
            siblings_count,
            getattr(instance, "products_count", 0),
            getattr(instance, "products_cumulative_count", 0),
        )

    category_analytics.short_description = _("Category Analytics")

    def products_count_display(self, instance):
        count = getattr(instance, "products_count", 0)
        return format_html(
            '<span class="inline-flex items-center px-2 py-1 text-xs font-semibold bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-300 rounded-full">{}</span>',
            count,
        )

    products_count_display.short_description = _("Direct Products")

    def recursive_products_display(self, instance):
        count = getattr(instance, "products_cumulative_count", 0)
        return format_html(
            '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-purple-50 dark:bg-purple-900 text-purple-700 dark:text-purple-300 rounded-full">{}</span>',
            count,
        )

    recursive_products_display.short_description = _("Total Products")


@admin.register(ProductReview)
class ProductReviewAdmin(ModelAdmin, TranslatableAdmin):
    compressed_fields = True
    list_fullwidth = True
    list_filter_sheet = True

    list_display = [
        "review_info",
        "product_link",
        "user_link",
        "rating_display",
        "status_badge",
        "created_display",
    ]
    list_filter = [
        "status",
        ("rate", SliderNumericFilter),
        ("created_at", RangeDateTimeFilter),
        ("product", RelatedDropdownFilter),
        ("user", RelatedDropdownFilter),
    ]
    actions = [
        "approve_reviews",
        "reject_reviews",
        "make_published",
        "make_unpublished",
    ]
    search_fields = [
        "translations__comment",
        "user__email",
        "user__username",
        "product__translations__name",
    ]
    list_select_related = ["product", "user"]
    readonly_fields = ("created_at", "updated_at", "uuid")
    list_filter_submit = True

    fieldsets = (
        (
            _("Review Information"),
            {
                "fields": ("product", "user", "rate", "status"),
                "classes": ("wide",),
            },
        ),
        (
            _("Content"),
            {
                "fields": ("comment",),
                "classes": ("wide",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("uuid", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def review_info(self, obj):
        comment = (
            obj.safe_translation_getter("comment", any_language=True)
            or "No comment"
        )
        comment_preview = (
            comment[:100] + "..." if len(comment) > 100 else comment
        )

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">Review #{}</div>'
            '<div class="text-base-600 dark:text-base-400">{}</div>'
            "</div>",
            obj.id,
            comment_preview,
        )

    review_info.short_description = _("Review")

    def product_link(self, obj):
        if obj.product:
            name = obj.product.safe_translation_getter(
                "name", any_language=True
            ) or str(obj.product.id)
            url = f"/admin/product/product/{obj.product.id}/change/"
            return format_html(
                '<div class="text-sm">'
                '<a href="{}" class="font-medium text-blue-600 dark:text-blue-400 hover:text-blue-800 dark:hover:text-blue-300">{}</a>'
                '<div class="text-base-500 dark:text-base-400">#{}</div>'
                "</div>",
                url,
                name,
                obj.product.id,
            )
        return "-"

    product_link.short_description = _("Product")

    def user_link(self, obj):
        if obj.user:
            url = f"/admin/user/useraccount/{obj.user.id}/change/"
            return format_html(
                '<div class="text-sm">'
                '<a href="{}" class="font-medium text-blue-600 dark:text-blue-400 hover:text-blue-800 dark:hover:text-blue-300">{}</a>'
                '<div class="text-base-500 dark:text-base-400">ID: {}</div>'
                "</div>",
                url,
                obj.user.email,
                obj.user.id,
            )
        return "-"

    user_link.short_description = _("User")

    def rating_display(self, obj):
        rate = obj.rate
        stars = "⭐" * rate + "☆" * (10 - rate)

        if rate >= 8:
            color = "text-green-600 dark:text-green-400"
        elif rate >= 6:
            color = "text-yellow-600 dark:text-yellow-400"
        else:
            color = "text-red-600 dark:text-red-400"

        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium {}">{}/10</div>'
            '<div class="text-xs">{}</div>'
            "</div>",
            color,
            rate,
            stars[:5],
        )

    rating_display.short_description = _("Rating")

    def status_badge(self, obj):
        status_config = {
            ReviewStatus.NEW: {
                "bg": "bg-blue-50 dark:bg-blue-900",
                "text": "text-blue-700 dark:text-blue-300",
                "icon": "🆕",
            },
            ReviewStatus.TRUE: {
                "bg": "bg-green-50 dark:bg-green-900",
                "text": "text-green-700 dark:text-green-300",
                "icon": "✅",
            },
            ReviewStatus.FALSE: {
                "bg": "bg-red-50 dark:bg-red-900",
                "text": "text-red-700 dark:text-red-300",
                "icon": "❌",
            },
        }

        config = status_config.get(
            obj.status,
            {
                "bg": "bg-gray-50 dark:bg-gray-900",
                "text": "text-base-700 dark:text-base-300",
                "icon": "❓",
            },
        )

        return format_html(
            '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium {} {} rounded-full gap-1">'
            "<span>{}</span>"
            "<span>{}</span>"
            "</span>",
            config["bg"],
            config["text"],
            config["icon"],
            obj.get_status_display(),
        )

    status_badge.short_description = _("Status")

    def created_display(self, obj):
        return format_html(
            '<div class="text-sm text-base-600 dark:text-base-400">{}</div>',
            obj.created_at.strftime("%Y-%m-%d %H:%M"),
        )

    created_display.short_description = _("Created")

    @action(
        description=_("Approve selected reviews"),
        variant=ActionVariant.SUCCESS,
        icon="check_circle",
    )
    def approve_reviews(self, request, queryset):
        updated = queryset.update(status=ReviewStatus.TRUE)
        self.message_user(
            request,
            ngettext(
                _("%(count)d review was approved."),
                _("%(count)d reviews were approved."),
                updated,
            )
            % {"count": updated},
            messages.SUCCESS,
        )

    @action(
        description=_("Reject selected reviews"),
        variant=ActionVariant.DANGER,
        icon="cancel",
    )
    def reject_reviews(self, request, queryset):
        updated = queryset.update(status=ReviewStatus.FALSE)
        self.message_user(
            request,
            ngettext(
                _("%(count)d review was rejected."),
                _("%(count)d reviews were rejected."),
                updated,
            )
            % {"count": updated},
            messages.SUCCESS,
        )

    def make_published(self, request, queryset):
        updated = queryset.update(status=ReviewStatus.TRUE)
        self.message_user(
            request,
            ngettext(
                _("%(count)d comment was successfully marked as published."),
                _("%(count)d comments were successfully marked as published."),
                updated,
            )
            % {"count": updated},
            messages.SUCCESS,
        )

    def make_unpublished(self, request, queryset):
        updated = queryset.update(status=ReviewStatus.FALSE)
        self.message_user(
            request,
            ngettext(
                _("%(count)d comment was successfully marked as unpublished."),
                _(
                    "%(count)d comments were successfully marked as unpublished."
                ),
                updated,
            )
            % {"count": updated},
            messages.SUCCESS,
        )


@admin.register(ProductFavourite)
class ProductFavouriteAdmin(ModelAdmin):
    compressed_fields = True
    list_fullwidth = True

    list_display = [
        "user_display",
        "product_display",
        "created_display",
    ]
    list_filter = [
        ("created_at", RangeDateTimeFilter),
        ("user", RelatedDropdownFilter),
        ("product", RelatedDropdownFilter),
        ("product__category", RelatedDropdownFilter),
    ]
    search_fields = [
        "user__email",
        "user__username",
        "product__translations__name",
        "product__sku",
    ]
    list_select_related = ["user", "product"]
    readonly_fields = ("created_at", "updated_at", "uuid")

    def user_display(self, obj):
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="text-base-500 dark:text-base-400">ID: {}</div>'
            "</div>",
            obj.user.email,
            obj.user.id,
        )

    user_display.short_description = _("User")

    def product_display(self, obj):
        name = (
            obj.product.safe_translation_getter("name", any_language=True)
            or "Unnamed Product"
        )
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="text-base-500 dark:text-base-400">#{}</div>'
            "</div>",
            name,
            obj.product.sku[:8],
        )

    product_display.short_description = _("Product")

    def created_display(self, obj):
        return format_html(
            '<div class="text-sm text-base-600 dark:text-base-400">{}</div>',
            obj.created_at.strftime("%Y-%m-%d %H:%M"),
        )

    created_display.short_description = _("Created")


@admin.register(ProductCategoryImage)
class ProductCategoryImageAdmin(ModelAdmin, TranslatableAdmin):
    compressed_fields = True
    list_fullwidth = True

    list_display = [
        "image_preview",
        "category_name",
        "image_type_badge",
        "status_badge",
        "sort_order",
        "created_at",
    ]
    list_filter = [
        "image_type",
        "active",
        ("category", RelatedDropdownFilter),
        ("created_at", RangeDateTimeFilter),
    ]
    search_fields = [
        "category__translations__name",
        "translations__title",
        "translations__alt_text",
    ]
    list_select_related = ["category"]
    readonly_fields = ("created_at", "updated_at", "uuid", "sort_order")
    ordering = ["category", "image_type", "sort_order"]

    fieldsets = (
        (
            _("Image Information"),
            {
                "fields": ("category", "image", "image_type", "active"),
                "classes": ("wide",),
            },
        ),
        (
            _("Content"),
            {
                "fields": ("title", "alt_text"),
                "classes": ("wide",),
            },
        ),
        (
            _("Organization"),
            {
                "fields": ("sort_order",),
                "classes": ("collapse",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("uuid", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def image_preview(self, obj):
        if obj.image:
            return format_html(
                '<img src="{}" style="width: 60px; height: 60px; object-fit: cover; border-radius: 8px; border: 1px solid #e5e7eb;" />',
                obj.image.url,
            )
        return format_html(
            '<div class="bg-gray-100 dark:bg-gray-900 rounded border border-gray-200 dark:border-gray-700 flex items-center justify-center text-base-400 dark:text-base-500 text-xs">No Image</div>'
        )

    image_preview.short_description = _("Preview")

    def category_name(self, obj):
        name = (
            obj.category.safe_translation_getter("name", any_language=True)
            or "Unnamed Category"
        )
        return format_html(
            '<div class="text-sm font-medium text-base-900 dark:text-base-100">{}</div>',
            name,
        )

    category_name.short_description = _("Category")

    def image_type_badge(self, obj):
        type_config = {
            "MAIN": {
                "bg": "bg-blue-50 dark:bg-blue-900",
                "text": "text-blue-700 dark:text-blue-200",
                "icon": "🖼️",
            },
            "BANNER": {
                "bg": "bg-purple-50 dark:bg-purple-900",
                "text": "text-purple-700 dark:text-purple-200",
                "icon": "🏞️",
            },
            "ICON": {
                "bg": "bg-green-50 dark:bg-green-900",
                "text": "text-green-700 dark:text-green-200",
                "icon": "🎯",
            },
        }

        config = type_config.get(
            obj.image_type,
            {
                "bg": "bg-gray-50 dark:bg-gray-900",
                "text": "text-base-700 dark:text-base-700",
                "icon": "📸",
            },
        )

        return format_html(
            '<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium {} {} rounded-full gap-1">'
            "<span>{}</span>"
            "<span>{}</span>"
            "</span>",
            config["bg"],
            config["text"],
            config["icon"],
            obj.get_image_type_display(),
        )

    image_type_badge.short_description = _("Type")

    def status_badge(self, obj):
        if obj.active:
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-300 rounded-full">'
                "✅ Active"
                "</span>"
            )
        else:
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-300 rounded-full">'
                "❌ Inactive"
                "</span>"
            )

    status_badge.short_description = _("Status")


@admin.register(ProductImage)
class ProductImageAdmin(ModelAdmin, TranslatableAdmin):
    compressed_fields = True
    list_fullwidth = True

    list_display = [
        "image_preview",
        "product_name",
        "main_badge",
        "sort_order",
        "created_at",
    ]
    list_filter = [
        "is_main",
        ("product", RelatedDropdownFilter),
        ("created_at", RangeDateTimeFilter),
    ]
    search_fields = [
        "product__translations__name",
        "product__sku",
        "translations__title",
    ]
    list_select_related = ["product"]
    readonly_fields = ("created_at", "updated_at", "uuid", "sort_order")
    ordering = ["product", "-is_main", "sort_order"]

    fieldsets = (
        (
            _("Image Information"),
            {
                "fields": ("product", "image", "is_main"),
                "classes": ("wide",),
            },
        ),
        (
            _("Content"),
            {
                "fields": ("title",),
                "classes": ("wide",),
            },
        ),
        (
            _("Organization"),
            {
                "fields": ("sort_order",),
                "classes": ("collapse",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("uuid", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def image_preview(self, obj):
        if obj.image:
            return format_html(
                '<img src="{}" style="width: 60px; height: 60px; object-fit: cover; border-radius: 8px; border: 1px solid #e5e7eb;" />',
                obj.image.url,
            )
        return format_html(
            '<div class="bg-gray-100 dark:bg-gray-900 rounded border border-gray-200 dark:border-gray-700 flex items-center justify-center text-base-400 dark:text-base-500 text-xs">No Image</div>'
        )

    image_preview.short_description = _("Preview")

    def product_name(self, obj):
        name = (
            obj.product.safe_translation_getter("name", any_language=True)
            or "Unnamed Product"
        )
        return format_html(
            '<div class="text-sm">'
            '<div class="font-medium text-base-900 dark:text-base-100">{}</div>'
            '<div class="text-base-500 dark:text-base-400">#{}</div>'
            "</div>",
            name,
            obj.product.sku[:8],
        )

    product_name.short_description = _("Product")

    def main_badge(self, obj):
        if obj.is_main:
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-200 rounded-full">'
                "⭐ Main"
                "</span>"
            )
        else:
            return format_html(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium bg-gray-50 dark:bg-gray-900 text-base-700 dark:text-base-700 rounded-full">'
                "📷 Gallery"
                "</span>"
            )

    main_badge.short_description = _("Type")
